/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       config.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       25-Feb-2013 12.15.47 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Read some info from configuration file                                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/
#include "config.h"

list<FS_control *>  Config::FS_control_list;
string Config::info_pathname       = "/etc/dcheck.conf";


Config::Config()
{
//cout << "Config::Config" << endl;
}

Config::~Config()
{
//cout << "Config::~Config" << endl;
}



/***************************
 * Load a file into memory *
 ***************************/
string Config::load_file(string pathname)
{
size_t size = 0; // This is a long long
ssize_t len;
int fd;
string conf_file;
char *buffer = NULL;

//cout << "dcheck: Config::load_file - ENTERED" << endl;
	
misc::get_fsize(pathname, &size);	
if (!size) return("");

if ( (buffer = (char *)calloc(size + 1, 1)) ) /* include zero byte */
	{
	/* Open the file and load it into buffer */
	fd = open(pathname.c_str(), O_RDONLY);
	if (fd >= 0)
		{
		len = read(fd, buffer, size);
		if (len == size)
			{
			conf_file = buffer;
			close(fd);
			free(buffer);
			return conf_file;
			}
		}
	}
if (buffer) free(buffer);
return conf_file;
}





char *Config::get_right(char *c, char *old_end)
{
char *end;
gboolean in_quote = FALSE;

/* Remove leading white space */
while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */

/* Skip an assignment character */
if ((*c == '=') || (*c == ':'))
	{
	c++;

	/* Skip white space after assignment character */
	while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */
    }

/* Now look for end of line starting here, or start of a comment */
end = c;

/*
 Find the end, comments are # or ; (Assumes no white space in the value and line ends with newline)
If old_end is defined, the character is the old character at the end which is replaced by the zero
*/
	
while ( (*end) && ( (in_quote && (*end != '\n')) ||
		(!in_quote && (*end != ' ') && (*end != '\t') && (*end != '#') && (*end != ';') && (*end != '\n') ) ) )
	{
//	g_print("GOT %c : in_quote=%d\n", *end, in_quote);
	if (*end == '\"')
		{
		if (in_quote && (*(end+1) == '\"')) end++;  /* literal */
		else in_quote ^= 1;
		}
	end++;
	}
  
if (old_end) *old_end = *end;  /* The character that was replaced by a zero */
*end = 0;

return(c);
}




void Config::add_to_control_list(string name, string control)
{
string n = name;
string c = control;	
FS_control *fsc = new(FS_control);				
fsc->name = n;
fsc->control = c;
	
FS_control_list.push_back(fsc);

}


/*********************************
 * Load configuration from file. *
 *********************************/

void Config::load_conf(void)
{
string name, control;
char *c;   // current position
char end;
string config_file = load_file(info_pathname);

su_auth = TRUE;                 // default
status_wanted = TRUE;           // default

//cout << "ENTERED Config::load_conf: config_file=\n" << config_file << endl;

if (!config_file.empty())
  {
  // Now parse it
	c = (char *)config_file.c_str();

	while (c && *c)
		{
		while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */

    if ( (*c == '#') || (*c == ';') || (*c == '\n') )
			{
			/* comments, newlines */
// cout << "Config::load_conf - FOUND COMMENT or NEWLINE\n";

			while (*c && *c != '\n') c++;
			if (*c) c++;
			}

		else if ( (strncasecmp(c, "FILESYSTEM", 10)==0) && ( (*(c+11)==' ') || (*(c+11)=='=')) )
			{
      name.clear();
			control.clear();


			c += 10;
	  	c = get_right(c, &end);

			if (*c)
				{
        name = c;
        }
			c += strlen(c);
			*c = end;
          

//cout << "Config::load_conf - FOUND FILESYSTEM name=" << name << endl;

      if (name.length())
        {
				control = "tune2fs"; // default

				if (c && *c)
					{
					while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */

  				if ( (*c == '#') || (*c == ';') || (*c == '\n') )
						{
						// comments, newlines
	//					cout << "Config::load_conf - FOUND COMMENT or NEWLINE 2\n";

						while (*c && *c != '\n') c++;
						if (*c) c++;
						}

					if ( (strncasecmp(c, "CONTROL", 7)==0) && ( (*(c+8)==' ') || (*(c+8)=='=')) )
  					{        
  					c += 7;
  					c = get_right(c, &end);
 
						if (*c)
							{
      				control = c;

 	//						cout << "Config::load_conf - FOUND CONTROL "<< control << endl;
     					}
						c += strlen(c);
						*c = end;
						}
					}



//			cout << "Config::load_conf - LISTING name= " << name <<" control= " << control << endl;

				add_to_control_list(name, control);
				}

			} 

		else if (
		//        ( (strncasecmp(c, "FILESYSTEM", 10)==0) && ( (*(c+11)==' ') || (*(c+11)=='=')) ) ||
		//				( (strncasecmp(c, "CONTROL", 7)==0) && ( (*(c+8)==' ') || (*(c+8)=='=')) ) ||
		        ( (strncasecmp(c, "STATUS_WANTED", 13)==0) && ( (*(c+14)==' ') || (*(c+14)=='=')) )
		        )
			{
			// SKIP THESE
			while (*c && (*c != '\n')) c++;
			}

    // Only   AUTHORISATION  is wanted here
    else if ( (strncasecmp(c, "AUTHORISATION", 13)==0) && ( (*(c+14)==' ') || (*(c+14)=='=')) )
			{
 //cout << "Config::load_conf - FOUND AUTHORISATION\n";
      c += 13;
      c = get_right(c, &end);
      if (*c)
        {
//cout << "Config::load_conf - AUTHORISATION is \"" << c << "\"" << endl;
					
        if ( strcasecmp(c, "sudo")==0 ) su_auth = FALSE;
			  c += strlen(c);
			  *c = end;
        }
      }
    else
			{
			/* UNRECOGNIZED, ignore to eol */
 			cout << "Config::load_conf - UNRECOGNIZED " << c << endl;
				
			while (*c && (*c != '\n')) c++;
			}

//		cout <<"Config::load_conf: end of loop, c= " << c << endl;

		} // END OF while (c && *c)
  }
//cout <<"Config::load_conf: Leaving : su_auth= " << su_auth << endl;

return;
}



// MUST be root to get here

void Config::fix_dcheck(void)
{
string result;
string command;

//cout << "Config::fix_dcheck - ENTERED" << endl;

// Get prefs
load_conf();
  

// Iterate through the filesystems
//  and fix those that are under dcheck control
FS_control *fsc;
list<FS_control *>::iterator i;
			
for (i=FS_control_list.begin(); i!=FS_control_list.end(); ++i)
  {
	fsc = (FS_control *)*i;

  if (fsc->control == "dcheck")
    {
    // GOT ONE, so fix it
    // create the command for this filesystem
		command = (string)"tune2fs -C 1 -c -1 " + (string)fsc->name;

		result = misc::get_output(command);

		// Not much point in checking the result...
    }
  }
}


