/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       main.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       24-Feb-2013 21.56.57 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* The entry point and GUI                                                   *
* If not run as root, get root priveleges using vtesu.c                     *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <errno.h>
#include "config.h"
#include "vtesu.h"

#include "misc.h"
using namespace misc;


Config config;
Vtesu vsu;





//////////////////////////////////////////////////////////////////////////////
///////////                                                 //////////////////
///////////                     START                       //////////////////
///////////                                                 //////////////////
//////////////////////////////////////////////////////////////////////////////

// Note: possibly use fflush() and ignore errors perhaps
static bool is_terminal(void)
{

// Can we receive input from a tty?
// If so, it was a terminal start so ignore the first "Return" character
// which is was what made the program run
if ( isatty(fileno(stdin)) )
	{
	//cout << "main: is_terminal - GOT a terminal" << endl; 

	fflush(stdin); // get rid of extraneous junk in stdin
	return TRUE;
	}
//cout << "main: is_terminal - NOT GOT a terminal" << endl; 
return FALSE;
}

string get_pathname_from_path(char *cmd)
{
string pathname;
struct stat sb;
char *c;
char *path = getenv("PATH");

c = path;

while (c && *c)
	{
	pathname.clear();

	while (*c && (*c != ':'))
		{
		pathname += *c;
		c++;
		}
	pathname += (string)"/" + cmd;

	if (!stat(pathname.c_str(), &sb)) // if it exists
		{
		return pathname;
		}
	if (*c == ':') c++;
	}
	
return "";
}

// Get main program's full pathname
string get_main_pathname(char *cmd)
{
string pathname;

// Get this program's full pathname
if (cmd[0] == '/') // if absolute
	{
		// get the real path
//		cout << "main: getting realpath (absolute)" << endl;
		char *path = realpath(cmd, NULL);
		if (path) pathname = path;
//		else
//			{
//			cout << "main: " << cmd << " not found: " << strerror(errno) << "\nGoodbye world"<< endl;
//			exit(EXIT_FAILURE);
//			}
	}
else if (!(cmd[0] == '.')) // if not relative
	{
	pathname = get_pathname_from_path(cmd);
	}

else // relative, get current directory and tag cmd onto it then realpath
	{
//	cout << "main: getting realpath (relative)" << endl;
	char *dir = getcwd(NULL,0);
//	cout << "main: getting realpath (relative) - dir= \"" << dir << "\"" << endl;
	pathname = (string)dir + "/" + cmd;
	free(dir);
//	cout << "main: 2- cmd= " << cmd << endl;
	char *path = realpath(pathname.c_str(), NULL);
	if (path) pathname = path;
	else pathname.clear();
//		{
//		cout << "main: " << cmd << " not found: " << strerror(errno) << "\nGoodbye world"<< endl;
//		exit(EXIT_FAILURE);
//		}			
	}


if (!pathname.empty()) pathname += "_main";
else pathname = (string)cmd + "_main"; // If all else fails!
return pathname;
}




// Our signal trap
void trap(int signum)
{
switch (signum)
  {
/*
  case SIGPOLL:   // not implemented
	break;

  case SIGUSR1:   // not implemented
	break;

  case SIGUSR2:   // not implemented
	break;

  case SIGTERM:   // not implemented
	break;
*/

    case SIGINT:  // QUIT
    default:
      {
			cout << "dcheck is terminating, signal " << signum << " (SIGINT)\nGoodbye world\n";
			exit(EXIT_SUCCESS);
      }
    }
}




int main(int argc, char *argv[])
{
bool fix_wanted = FALSE;

is_terminal();

string cmd = get_main_pathname(argv[0]);
//	
//for (int i = 0; i< argc; ++i)
//	{
//	cout << "argv[" << i << "]= " << argv[i] << endl;
//	}

	
//cout << "main : ENTERED - cmd= " << cmd << endl;

struct sigaction act;

act.sa_handler = trap;
sigemptyset(&act.sa_mask);
act.sa_flags = 0;
sigaction(SIGINT, &act, 0);     // 2 Interrupt (ANSI)

  //signal(SIGTERM, Trap);
  //signal(SIGINT,  Trap);
  //signal(SIGUSR1, Trap);
  //signal(SIGUSR2, Trap);
  //signal(SIGPOLL, Trap);

  //sigaction(SIGTERM, &act, 0);    // 15 Termination (ANSI).
  //sigaction(SIGUSR1, &act, 0);    // 10 User-defined signal 1 (POSIX).
  //sigaction(SIGUSR2, &act, 0);    // 12 User-defined signal 2 (POSIX).
  //sigaction(SIGPOLL, &act, 0);    // 29 Pollable event occurred (System V).



cout << "Hello world" << endl;

if (!gtk_init_check(&argc, &argv))
  {
  // Can't get a GUI, so run it as "fix" if user is root
		
  if ( (strcmp( (getpwuid( getuid() ))->pw_name , "root" )) ) // if we are not root
    {
    cout <<"Can't run - You must be root if no GUI!\nGoodbye world" << endl;   
    exit(EXIT_FAILURE);
    }

  config.fix_dcheck();  // We are root without a GUI so fix things up

	cout << "Goodbye world" << endl;

  exit(EXIT_SUCCESS);
  }
		

// Check that command dcheck_main is available - stat it...
struct stat sb;
if (stat(cmd.c_str(), &sb))
	{
	// stat failed
	cout << "Can't find \"" << cmd << "\"" << endl;
	cout << "Goodbye world" << endl;
	exit(EXIT_FAILURE);
	}
    
config.load_conf();
//cout << "main : DONE config.load_conf : config.su_auth= " << config.su_auth << endl;


// Parse args to FIX dcheck if requested
if ( ( (argc)==2 ) && ( (STRIEQ( (argv[1]), "-f") ) || (STRIEQ( (argv[1]), "--fix") ) ) )
  {
	fix_wanted = TRUE;
  }
  

if ( (strcmp( (getpwuid( getuid() ))->pw_name , "root" )) ) // if we are not root
  {
	//cmd = "../dcheck_main/dcheck_main"; // for now...  

		
		// NOTE: terminal command "which" gets a full pathname from PATH
		// Could use something like get_output(command = "which "+ cmd);


		
//	cout << "main : config.su_auth= " << config.su_auth << endl;

	vsu.su_auth	= config.su_auth;
	vsu.fix_wanted = fix_wanted;
	vsu.start_pid = getpid();
		
//	cout << "main : Using " << ((vsu.su_auth) ? "-su" : "-sudo") << endl;
//	cout << "main : cmd= \"" << cmd  << "\" : length= " << cmd.length() << endl;

// TESTING
//	cmd = "../dcheck_main/dcheck_main"; 
		
	char *array[] =
    {
    (char *)cmd.c_str(),                          // The program to execute as root - MAKE IT dcheck_main
		(char *)((config.su_auth) ? "-su" : "-sudo"), // su or sudo
		(char *)"-ni",                                // omit the ignore button
  	(char *)NULL                                  // terminate...
  	};
    
  vsu.vtesu(4, array);


	for (;;) { sleep(1); }


// When dcheck quits, it should send SIGINT to be caught by this process
// i.e. it shouldn't get here

	cout << "Goodbye world" << endl;
	exit (EXIT_SUCCESS);
  }

//cout << "main: There is a GUI" << endl;
	
  
// =============================================
// If we get here we are root and there is a GUI
// =============================================

// Parse args to FIX dcheck if requested
if ( ( (argc)==2 ) && ( (strcasecmp( (argv[1]), "-f")==0 ) || (strcasecmp( (argv[1]), "--fix")==0 ) ) )
  {
  config.fix_dcheck();
  }
  
//cout << "main: getuid = " << getuid() << endl;
//cout << "main: geteuid= " << geteuid() << endl;
//cout << "main: getgid = " << getgid() << endl;
//cout << "main: getegid= " << getegid() << endl;



//////////////////////////////////////////////////////////////////////////////
///////////                                                 //////////////////
///////////      Start dcheck_main here                     //////////////////
///////////                                                 //////////////////
//////////////////////////////////////////////////////////////////////////////

pid_t pid = 0;
//cmd = "su - -c gedit"; // for now...

//cmd = "gedit";

cmd = "sudo " + cmd;  // Make sure it runs with full root priveleges


//string thread_command = "vsu.new_process(" + cmd + ")";
// To get this to work it may be necessary to place new_process() into misc.
// or try using system...

int rc = start_thread( (char *)cmd.c_str() );

if (rc)
	{
  cout << "dcheck: main: ERROR - return code from pthread_create() is %d\n" << rc;
  exit(EXIT_FAILURE);
	}
	
// This doesn't return until the new process exits...
// A thread might work.

				
//vsu.new_process(cmd.c_str());  // This doesn't pass the pid
sleep(1);          // Give it time to start...


vsu.check_running(cmd, getpid(), &pid);

//cout << "main: this pid= " << getpid() << " : returned pid= " << pid << endl;


if (!vsu.check_running(cmd, getpid(), &pid))
	{
	// Command didn't start
	cout << "Already root but command \"" << cmd << "\" failed" << endl; 
	cout << "Goodbye world" << endl;
	exit (EXIT_FAILURE);
	}

while (vsu.is_running(pid, TRUE)) { sleep(1);}


cout << "Goodbye world" << endl;


exit(EXIT_SUCCESS);
}


