/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       misc.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       25-Feb-2013 12.02.50 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Various functions used in common                                          *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <pthread.h>
 
#include "misc.h"

namespace misc
{

/************************************
 * Execute a command using "system" *
 ************************************/
	
void *exec_system(void *command)
{
system((char *)command);
}


/**********************
 * Start a new thread *
 **********************/

int start_thread(char *command)
{
pthread_t thread;
return(pthread_create(&thread, NULL, exec_system, (void *)command));
}


/********************************
 * Return length of file fname  *
 * in bytes.                    *
 * Return FALSE if it fails.    *
 ********************************/

bool get_fsize(string fname, size_t *size)
{
    struct stat statbuf;

    if (stat(fname.c_str(), &statbuf) == 0)
		{
		*size = statbuf.st_size;
		return(TRUE);
		}
return(FALSE);
}

/***************************
 * Load a file into memory *
 ***************************/
char *load_file(char *pathname)
{
size_t size = 0; // This is a long long
size_t len;
int fd;
char *buffer = NULL;
	
get_fsize(pathname, &size);	
if (!size) return(NULL);

if ( (buffer = (char *)calloc(size + 1, 1)) ) /* include zero byte */
	{
	/* Open the file and load it into buffer */
	fd = open(pathname, O_RDONLY);
	if (fd >= 0)
		{
		len = read(fd, buffer, size);
		if (len == size)
			{
			close(fd);
			return(buffer);
			}
		}
	}
if (buffer) free(buffer);
return(NULL);
}


/******************************************
 * Pipe output of a command into a string *
 ******************************************/

string get_output(string command)
{
FILE *fp;
string output;;

char pipebuffer[PIPE_BUF];

if ( (fp = popen(command.c_str(), "r")) )
	{
	while ( fgets(pipebuffer, PIPE_BUF, fp) )
		{
    output += pipebuffer;
		}
  if (ferror(fp))
    {
    // ERROR CONDITION, returns empty string
    output.clear();
		}
	}

else
	  {
	  // ERROR: can't open pipe, returns empty string
	  }

pclose(fp);
return(output); // Either error or no output if empty...
}





/*************************************
 * Create a spinner (integer values) *
 *************************************/
GtkWidget *create_spinner(int s_start, int s_min, int s_max)
{
GtkAdjustment *spinner_adj;
spinner_adj = (GtkAdjustment *) gtk_adjustment_new (s_start, s_min, s_max, 1.0, 0.0, 0.0);
// creates the spinner, with no decimal places
return( gtk_spin_button_new (spinner_adj, 1.0, 0) );
}

/*************************************
 * Convert a long double to string   *
 * The string will ne changed by     *
 * subsequent calls                  *
 *************************************/
// I want to output a decimal string with no more that 2 chars after the decimal point...
string convert_f_to_string(double val)
{
static char result[1024]; // should be long enough!

snprintf(result, 1024, "%.02f", val); // the size includes the NULL byte
return result;
}



} // End of namespace misc
