/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       vtesu.cpp                                                 *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       24-Feb-2013 13.11.27 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* This is the GUI used to invoke su or sudo                                 *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

// #define _GNU_SOURCE

#include <stdio.h>

#include <string.h>
#include <sys/wait.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

// NB: some systems may use this instead:-
//#include <vte/vte.h>
#include <vte-0.0/vte/vte.h>

#include "vtesu.h"



GdkRGBA Vtesu::red   = {1, 0, 0, 1};



Vtesu::Vtesu(void)
{
//cout << "Vtesu::Vtesu" << endl;
}

Vtesu::~Vtesu(void)
{
//cout << "Vtesu::~Vtesu" << endl;
}

void Vtesu::callback_destroy(GtkWidget *widget, Vtesu *vsu)
{
//cout << "Vtesu::callback_destroy" << endl;

if (vsu->pixbuf)
	{
	g_object_unref(G_OBJECT(vsu->pixbuf));	
	vsu->pixbuf = NULL;
//	cout << "Vtesu::callback_destroy - Done g_object_unref" << endl;
	}

//cout << "Vtesu::callback_destroy - ABOUT to gtk_main_quit" << endl;
	
gtk_main_quit();

}

void Vtesu::callback_quit(GtkWidget *widget, Vtesu *vsu)
{
//cout << "Vtesu::callback_quit" << endl;
	
gtk_widget_destroy(vsu->window);
	
//cout << "Vtesu::callback_quit _ DONE gtk_widget_destroy(vsu->window)" << endl;

}



/****************************************************
 * Ignore - try to execute as user instead of root  *
 ****************************************************/

void Vtesu::callback_ignore(GtkWidget *widget, Vtesu *vsu)
{
string cmd = vsu->command + " &\n";
gtk_label_set_text(GTK_LABEL(vsu->status_label), "");
pid_t pid;

while (gtk_events_pending ())
			{
			gtk_main_iteration ();
			}


//------------------------------------------------------------------------
// NOTE: It may be possible to use "system(cmd.c_str()) instead of this.
//       Check it out later.
//------------------------------------------------------------------------
vsu->new_process( cmd.c_str() );


// ====================================
// Now check if the command is running
// ====================================

if (vsu->check_running(vsu->command, vsu->shell_pid, &pid))
  {
//   cout << "Vtesu::callback_ignore - command is running" << endl;


  gtk_widget_hide (vsu->window);

	while (gtk_events_pending ())
		{
		gtk_main_iteration ();
		}
	  
// 	cout << "Vtesu::callback_ignore :  DONE gtk_widget_hide (window)" << endl;
 
	while (vsu->is_running(pid, FALSE))
		{
		sleep(1);
		}

  callback_quit(widget, vsu);
  }
else
  {
  // Command didn't start
//  cout << "callback_ok - command is NOT running" << endl;
  gtk_label_set_text(GTK_LABEL(vsu->status_label), "Failed to run the command... Retry!");
  }
return;
}


/******************************************
 * Start a new process running "command". *
 ******************************************/

int Vtesu::new_process(const char *command)
{

extern char **environ;


pid_t pid;

// NOTE:  pipes -  write into fp[1] and read from fp[0]

if (! (command && *command) ) return(-1);  // NO COMMAND

//  if (pipe(fp)) return(-2);   // NOT USING PIPES

if ((pid = fork()) == -1) return(-2);      // FORK FAILED

if (pid == 0)
  {

  //==============
  // CHILD PROCESS
  //==============

  char *argv[4];
  argv[0] = (char *)"sh";
  argv[1] = (char *)"-c";
  argv[2] = (char *)command;
  argv[3] = 0;

//  cout << "CHILD PROCESS - command= " << command << endl;

  execve("/bin/sh", argv, environ);

  exit(127);                          // Error if it returns...
  }

//===============
// PARENT PROCESS
//===============

//cout << "PARENT PROCESS - command= " << command << endl;

sleep(1);    // give it a moment to start



for(;;)
  {
  int status = 0;

  if (waitpid(pid, &status, 0) == -1)
    {
    if (! (errno == EINTR) ) return(-3);
    }
  else if (WIFEXITED(status))
    {
    status = (WEXITSTATUS(status));


    return(status);
    }
  else
    {

    return(-4);  // UNKNOWN ERROR
    }
  }
}




/*************************************
 * check if process pid is running   *
 *************************************/

bool Vtesu::is_running(pid_t pid, bool as_root)
{
// Get a list of running pids
string plist= misc::get_output(
	(  (as_root) ?  (char *)"ps -eo \"%p\" --no-headers" :
									(char *)"ps -eo \"%p\" --no-headers" ) );

                               
char *c = (char *)plist.c_str();
char *endptr;
int val;

while (*c)
	{
	val = strtol(c, &endptr, 10);

	if ( pid == val)
		{
		return TRUE;
		}
	c = endptr;
	while ( (*c) && ( (*c) != '\n') )
		{
		c++;
		}
	if (*c) c++;
	}
return FALSE;
}




/*****************************************
 * Check whether the command is running, *
 * starting at start_pid. If pid is not  * 
 * NULL, put command's pid into it.      *
 *****************************************/

bool Vtesu::check_running(string cmd, pid_t start_pid, pid_t *pid)
{
char *c;
int val;
char *endptr;
int n;
	
*pid = 0;  // no pid yet

for (n=10; n; --n)
  {
  string plist = misc::get_output( (char *)"ps -eo \"%p %a\" --no-headers" );

// cout << "Vtesu::check_running: plist= \n" << plist << endl;
// cout << "Vtesu::check_running: shell_pid= " << shell_pid << endl;
// cout << "Vtesu::check_running:       cmd= " << cmd << endl;

  c = (char *)plist.c_str();

  while (*c)
    {
    if (atoi(c) == start_pid)  // The place to start searching
      {
// 			cout << "Vtesu::check_running: GOT start_pid" << endl;

      break;
      }
    while ( (*c) && ( (*c) != '\n') )
      {
      c++;
      }
    if (*c) c++;
    }

// Now look for the command

//cout << "Vtesu::check_running: c= \n" << c << endl;

	int i = 1;

  while (*c)
    {
// 		cout << "Vtesu::check_running " << i++ << " (1): c= \n" << c << endl;

    val = strtol(c, &endptr, 10);
    c = endptr;

    while (isblank(*c)) c++;

// 		cout << "Vtesu::check_running " << i << " (2): c= \n" << c << endl;
// 		cout << "Vtesu::check_running - cmd.length()=" << cmd.length() << endl;
// 		cout << "Vtesu::check_running - strlen(cmd.c_str())=" << strlen(cmd.c_str()) << endl;


    if (strncmp( c, cmd.c_str(), cmd.length() )==0)
      {
//      cout << "Vtesu::check_running -c= " << c << " : cmd= " << cmd << endl;
      if (pid)  // pid of this command
        {
        *pid = val;
        }

      return TRUE;
      }


    while ( (*c) && ( (*c) != '\n') )
      {
      c++;
      }
    if (*c) c++;
    }
		
		usleep(200000); // small pause and try again up to 10 times
		} // for (int n=10; n; --n)

// FAILED
return FALSE;
}





/************************************
 * OK button or "Enter key pressed  *
 ************************************/

void Vtesu::callback_ok(GtkWidget *widget, Vtesu *vsu)
{
string cmd;
string this_command;
int command_pid;

//cout << "Vtesu::callback_ok" << endl;

string password = (char *)gtk_entry_get_text(GTK_ENTRY(vsu->password_entry));
password += "\n";
  
// vte_terminal_set_color_foreground(VTE_TERMINAL(vsu->vte), &black);
// vte_terminal_set_color_background(VTE_TERMINAL(vsu->vte), &white);

gtk_label_set_text(GTK_LABEL(vsu->status_label), "");

while (gtk_events_pending ())
	{
  gtk_main_iteration ();
	}

switch (vsu->start_type)
  {
  case SUDO: // OK
    {
    this_command = vsu->command;
    cmd ="sudo -b " + vsu->command + "\n";
    vte_terminal_feed_child(VTE_TERMINAL(vsu->vte), cmd.c_str(), -1);

    sleep(1);

    vte_terminal_feed_child(VTE_TERMINAL(vsu->vte), password.c_str(), -1);
 
    break;
    }

  
  case SU: // OK
  default:
    {
// 		cout << "Vtesu::callback_ok - GOT SU" << endl;
// 		cout << "Vtesu::callback_ok - vsu->command= " << vsu->command << endl;
	
	//		this_command = canonicalize_file_name(vsu->command.c_str());   // Sementation Fault!
	//	this_command = realpath(vsu->command.c_str(), NULL);           // Sementation Fault!

	//	if (this_command.empty()) cout << "Vtesu::callback_ok - realpath FAILED" << endl;
		this_command = (string)"su - -c " + vsu->command;
			
    cmd = (string)"su - -c \"" + vsu->command + "\"\n";              

// 		cout << "Vtesu::callback_ok - cmd= " << cmd << endl;
			
    vte_terminal_feed_child(VTE_TERMINAL(vsu->vte), cmd.c_str(), -1);

    sleep(1);

    command_pid = 0;  
    if (! vsu->check_running(this_command, vsu->shell_pid, &command_pid))
      {
// 			cout << "Vtesu::callback_ok (vsu->check_running): this_command= " << this_command << endl;
// 			cout << "Vtesu::callback_ok (vsu->check_running):          cmd= " << cmd << endl;
// 			cout << "Vtesu::callback_ok (vsu->check_running):  command_pid= " << command_pid << endl;

      gtk_label_set_text(GTK_LABEL(vsu->status_label), "Failed to run the command... Retry! (su didn't start)");
      gtk_entry_set_text(GTK_ENTRY(vsu->password_entry), "");
      cmd.clear();
      password.clear();
      return;
      }

    vte_terminal_feed_child(VTE_TERMINAL(vsu->vte), password.c_str(), -1);

    break;
    }

  }

// Probably should overwrite password before clearing it...
for (int i=0; i<password.length(); ++i) { password[i] = ' '; }
password.clear();

if (vsu->check_running(vsu->command, vsu->shell_pid, &command_pid))
  {

	//cout << "Vtesu::callback_ok : vsu->command= " << vsu->command << " : command_pid= " << command_pid << endl;
      
  if (vsu->is_running(command_pid, TRUE))
    {
		//cout << "Vtesu::callback_ok : running as root\n";

    gtk_widget_hide (vsu->window);

		while (gtk_events_pending ())
			{
			gtk_main_iteration ();
			}
	  
		// cout << "Vtesu::callback_ok :  DONE gtk_widget_hide (window)" << endl;
 
		while (vsu->is_running(command_pid, TRUE))
			{
			sleep(1);
			}
	  
		// cout << "Vtesu::callback_ok 2: start_pid= " << vsu->start_pid << endl;
	
		kill(vsu->start_pid, SIGINT);  // This needs to be a signal that can be used by start_pid
			
    gtk_main_quit();

    }
  else
    {
    // Kill the command and inform the user about the problem
    if (kill(command_pid, SIGTERM)== -1)
      {
//      cout << "callback_ok - kill failed!" << endl;
      gtk_label_set_text(GTK_LABEL(vsu->status_label), "Authentication failed. The command ran but\nwithout root priveleges. Quit and Retry!");
      gtk_entry_set_text(GTK_ENTRY(vsu->password_entry), "");
      }
    else
      {
      gtk_label_set_text(GTK_LABEL(vsu->status_label), "Authentication failed. The command ran but\nwithout root priveleges and was killed.  Retry!");
      gtk_entry_set_text(GTK_ENTRY(vsu->password_entry), "");
      }
    }
  }
else
  {
  // Command didn't start
  string msg = ((vsu->start_type==SU) ? "su" : "sudo");

  msg += " -  started  but the command failed... Retry!";
    
  gtk_label_set_text(GTK_LABEL(vsu->status_label), msg.c_str());
  gtk_entry_set_text(GTK_ENTRY(vsu->password_entry), "");
  }
}



/******************
 * Parse our args *
 ******************/

bool Vtesu::parse_opts(int argc, char *argv[])
{
start_type           = SU;    // default
ignore_button_wanted = TRUE;  // default
int n;

error.clear();
title.clear();
message.clear();
command.clear();

//cout << "Vtesu::parse_opts - ENTERED" << endl;

for (n=0; (n<argc) && (argv[n]); ++n)
  {

//cout << "Vtesu::parse_opts 1 - argv[" << n << "]= " << argv[n] << endl;

// -t
  if (strncasecmp(argv[n], "-t", 2)==0)      // Use following title
    {
    if (title.length() == 0)
      {
      if (strlen(argv[n]) > 2)
        {
        title = &(argv[n])[2];      
        }
      else if ( (argc > n) && (argv[n+1][0] != '-') )
        {
        title = argv[++n];
        }
      else
        {
        // ERROR: -t but no title : not critical

 				error += "Vtesu::parse_opts - WARNING: -t but no title\n";
        return FALSE;
        }
      }
    else
      {
      // ERROR: title already defined

			error += "Vtesu::parse_opts - ERROR: -t but title has already been defined as \"" + title + "\"\n";
      return FALSE;
      }
    }

// -m
  else if (strncasecmp(argv[n], "-m", 2)==0)        // Use following message
    {
    char *msg = (char *)"";

    if (message.length()==0 )
      {
      if (strlen(argv[n]) > 2)
        {
        msg = &(argv[n])[2];
        }
      else if ((argc > n+1) && (argv[n+1][0] != '-'))
        {
        msg = argv[++n];
        }
      else
        {
        // ERROR: -m but no message : not critical
 				error += "Vtesu::parse_opts - WARNING: -m but no message\n";
        return FALSE;
        }

      char *c = msg;

      while (*c)
        {
        if (strncmp(c, "\\n", 2)==0)
          {
      		message += "\n";
          c++;
          }

        else if (strncmp(c, "\\t", 2)==0)
          {
     			message += "\t";
          c++;
          }

        else if (strncmp(c, "\\a", 2)==0)
          {
       		message += "\a";
          c++;
          }

        else if (strncmp(c, "\\b", 2)==0)
          {
					message += "\b";
          c++;
          }

        else if (strncmp(c, "\\f", 2)==0)
          {
 					message += "\f";
          c++;
          }
        else if (strncmp(c, "\\r", 2)==0)
          {
					message += "\r";	
          c++;
          }
        else if (strncmp(c, "\\v", 2)==0)
          {
					message += "\v";
          c++;
          }
         else if (strncmp(c, "\\", 2)==0)
          {
					message += "\\";
          c++;
          }
        else if (strncmp(c, "\\?", 2)==0)
          {
					message += "\?";
          c++;
          }
        else if (strncmp(c, "\\'", 2)==0)
          {
					message += "\'";
          c++;
          }
        else if (strncmp(c, "\\\"", 2)==0)
          {
					message += "\"";
          c++;
          }
        else
          {
          message += *c;  // Shouldn't this be *c? I think so!
          }
        c++;
        }
      }
    else
      {
      // ERROR: message already defined
			error += "Vtesu::parse_opts - ERROR: -m but message has already been defined as \"" + message + "\"\n";
			return FALSE;
      }
    }

// -c
  else if (strncasecmp(argv[n], "-c", 2)==0) // Use the following command
    {
    if (command.length() == 0)
      {      
      if (strlen(argv[n]) > 2)
        {			
				command = &(argv[n])[2];
        }


      while ( (argc > n+1) && (argv[n+1][0] != '-') )
        {
        n++;
				command += " " + *argv[n];
        }
      }
    else
      {
      // ERROR: command already defined
 			error += "Vtesu::parse_opts - ERROR: -c but command has already been defined as \"" + command + "\"\n";
      return FALSE;
      }
    }

// -su
  else if (strcasecmp(argv[n], "-su")==0)    // Use sudo
    {
		start_type = SU;
		}

// -sudo
  else if (strcasecmp(argv[n], "-sudo")==0)  // Use su
    {
		start_type = SUDO;
		}

// -ni
  else if (strcasecmp(argv[n], "-ni")==0)    // No ignore button
    {
    ignore_button_wanted = FALSE;
    }

// ERROR: unrecognised arg
  else if ((n<argc) && (argv[n][0] == '-'))
    {
    // ERROR: unrecognised arg

			
//cout << "Vtesu::parse_opts 2 ERROR: unrecognised arg- argv[" << n << "]= " << argv[n] << endl;

		error += (string)"Vtesu::parse_opts - ERROR: unrecognised arg \"" + argv[n];
		error +=	"\"\n";

		return FALSE;
    }
		
// default is command if none of the above
  else if (command.length() == 0)
    {
    command = argv[n];
    
    while ( ((n+1) < argc) && (argv[n+1][0] != '-') )
      {
			command += " " + *argv[++n];
      }
    }
  else
    {
    // ERROR: command already defined
		error += "Vtesu::parse_opts - ERROR: -c but command has already been defined as \"" + command + "\"\n";
		return FALSE;
    }

//	cout << "Vtesu::parse_opts END - argv[" << n << "]= " << argv[n] << endl;
  } // End of for

if (command.empty())
	{
	error += "Vtesu::parse_opts - ERROR: command not found\n";
	return FALSE;
	}
return TRUE;
}


/**************************
 * Check for "Return" key *
 **************************/

void Vtesu::key_event(GtkWidget *widget, GdkEventKey *event, Vtesu *vsu)
{

//cout << "Vtesu::key_event - gdk_keyval_name= " << gdk_keyval_name(event->keyval) << endl;

if ( (string)gdk_keyval_name(event->keyval) == "Return" )
	{
	vsu->callback_ok(NULL, vsu);
	}

return;
}



/*******************************
 * The GUI for root's password *
 *******************************/

// Note: to get here, user is NOT root
void Vtesu::vtesu(int argc, char *argv[])
{
GtkWidget *table;
GtkWidget *scrolled_window;
GtkWidget *wid, *buttonbox, *vbox;
long size;
char *buf;
char *ptr;

sleep(1);  // small delay needed

string user = (getpwuid( getuid() ))->pw_name;
string text;


// NOTE: gtk_init_check(&argc, &argv) already run in main.cpp

if ( (argc >= 1) && parse_opts(argc, argv) )
  {
  // cout << "Vtesu::vtesu 1" << endl;

	if (fix_wanted) command += " -f";
		
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_widget_set_size_request(GTK_WIDGET(window), 460, 248); //408, 248);    // width, height
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
  g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (callback_destroy),
		    this);
	g_signal_connect(window, "key-release-event", G_CALLBACK(key_event), this);

  gtk_container_set_border_width(GTK_CONTAINER(window), 0);

  if (title.length())
    {
    gtk_window_set_title(GTK_WINDOW(window), title.c_str());
    }
	//else cout << "Vtesu::vtesu: NO title" << endl;


	pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
	                                       GTK_STOCK_DIALOG_AUTHENTICATION,
	                                       GTK_ICON_SIZE_DIALOG);

	
  if (pixbuf)
		{
    gtk_window_set_icon( GTK_WINDOW(window), pixbuf);
		}
// 	else cout << "Vtesu::vtesu - NO pixbuf" << endl;

	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
	gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);
	gtk_container_add(GTK_CONTAINER(window), vbox);

  table = gtk_grid_new();
	gtk_grid_set_column_homogeneous(GTK_GRID(table), FALSE);
	gtk_grid_set_row_homogeneous(GTK_GRID(table), FALSE);
	gtk_grid_set_row_spacing (GTK_GRID(table), 10);
// 	cout << "Vtesu::vtesu - Created grid" << endl;
		
  gtk_container_add(GTK_CONTAINER(vbox), table);

  if ( message.empty() )
    {
    message = "Root priveleges have been requested. Using \"";
    switch(start_type)
      {
      case SU:
			message += "su - \".";
      break;

      case SUDO:
			message += "sudo \".";
      break;
      }
    }




  wid = gtk_image_new_from_stock(GTK_STOCK_DIALOG_AUTHENTICATION, GTK_ICON_SIZE_DIALOG);



	gtk_grid_attach(GTK_GRID(table), wid, 0, 0, 1, 1); // left, top, width, height

// 	cout << "Vtesu::vtesu - Attached image" << endl;

// Message
  wid = gtk_label_new(message.c_str());

	// cout << "Vtesu::vtesu - Got message label? " << ( (wid) ? "YES" : "NO") << endl;

  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5);

	gtk_grid_attach(GTK_GRID(table), wid, 0, 4, 10, 1); // left, top, width, height
		
	// cout << "Vtesu::vtesu - Attached message label" << endl;


		
// Command
	text = "Command:   " + command;
  wid = gtk_label_new(text.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5);

	gtk_grid_attach(GTK_GRID(table), wid, 0, 8, 12, 1); // left, top, width, height

	// cout << "Vtesu::vtesu - Attached command label - " << text << endl;

// Password
	text = "Password for ";
	text += (start_type == SUDO) ? user : "root";
	text += ":";

	wid = gtk_label_new(text.c_str());
  gtk_misc_set_alignment(GTK_MISC(wid), 0, .5);
  
	gtk_grid_attach(GTK_GRID(table), wid, 0, 12, 1, 1); // left, top, width, height


		
  password_entry = gtk_entry_new();
  text = "Password\nEnter ";
	text += (start_type == SUDO) ? user : "root";
	text +="'s password";


  gtk_entry_set_visibility(GTK_ENTRY(password_entry), FALSE);

	gtk_widget_set_tooltip_text(GTK_WIDGET(password_entry), text.c_str());

	gtk_grid_attach(GTK_GRID(table), GTK_WIDGET(password_entry), 1, 12, 11, 1); // left, top, width, height






		
// Status
  status_label = gtk_label_new("");
  gtk_misc_set_alignment(GTK_MISC (status_label), 0, 1);
	gtk_widget_override_color(GTK_WIDGET(status_label), GTK_STATE_FLAG_NORMAL, &red);
	gtk_grid_attach(GTK_GRID(table), status_label, 0, 16, 12, 1); // left, top, width, height

	// cout << "Vtesu::vtesu - Attached status label" << endl;

		
		
	buttonbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonbox), GTK_BUTTONBOX_EDGE);// GTK_BUTTONBOX_SPREAD);
  gtk_box_set_spacing(GTK_BOX(buttonbox), 0);



	gtk_box_pack_end(GTK_BOX(vbox), GTK_WIDGET(buttonbox), FALSE, FALSE, 0);


// OK BUTTON
  wid = gtk_button_new_with_label("OK, Do It");
  gtk_button_set_image(GTK_BUTTON(wid ), gtk_image_new_from_stock(GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON));


	g_signal_connect(G_OBJECT(wid), "clicked",
		    	G_CALLBACK(callback_ok),  this);
  gtk_container_add(GTK_CONTAINER(buttonbox), GTK_WIDGET(wid));


  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(wid))));
	gtk_widget_set_tooltip_text(GTK_WIDGET(wid), "OK, Do It\nUse the entered password");
		

// IGNORE BUTTON
//if (ignore_button_wanted)
    {
    wid = gtk_button_new_with_label("Ignore");
    gtk_button_set_image(GTK_BUTTON(wid ), gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON));
		gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(wid))));
		gtk_widget_set_tooltip_text(GTK_WIDGET(wid), "Ignore\nExecute the command without authorisation");

    g_signal_connect(G_OBJECT(wid), "clicked",
		    	G_CALLBACK(callback_ignore),  this);
 		gtk_container_add(GTK_CONTAINER(buttonbox), GTK_WIDGET(wid));
	
    }
		
// CANCEL BUTTON

  wid = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
//  gtk_button_set_image(GTK_BUTTON(wid ), gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));
  g_signal_connect(G_OBJECT(wid), "clicked",
		    	G_CALLBACK(callback_quit),  this);
	gtk_widget_set_tooltip_text(GTK_WIDGET(wid), "Cancel\nTerminate the program");
	
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(wid))));
	gtk_container_add(GTK_CONTAINER(buttonbox), GTK_WIDGET(wid));


  gtk_widget_show_all(window);



// From here on should be invisible
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
																GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  
	gtk_grid_attach(GTK_GRID(table), scrolled_window, 10, 10,180, 110); // left, top, width, height


  vte = vte_terminal_new();
  vte_terminal_set_background_transparent(VTE_TERMINAL(vte), FALSE);
  vte_terminal_set_size(VTE_TERMINAL(vte), 80, 25);



  char **a = 0;
  g_shell_parse_argv("/bin/sh", 0, &a, 0);
  vte_terminal_fork_command_full(VTE_TERMINAL(vte), (VtePtyFlags)0, ptr, a, 0, (GSpawnFlags)0, 0, 0, &shell_pid, 0);

  vte_terminal_set_scroll_on_keystroke(VTE_TERMINAL (vte), TRUE);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), vte);


//  cout << "Vtesu::Vtesu - pid= " << pid << endl;  // This is the pid of /bin/sh if full, othewise /bin/bash


  gtk_main (); // START the main gtk loop

  }
else
  {
//	cout << "Vtesu::vtesu - error detected" << endl;

	cout << endl << error << endl;
  exit(EXIT_FAILURE);
  }


//g_print("Vtesu::vtsu : LEAVING\n");

exit(EXIT_SUCCESS);
}


