/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       config.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       04-Mar-2013 17.20.45 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Read /etc/dcheck.conf                                                     *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/
#include "config.h"

bool Config::su_auth;
bool Config::status_wanted;

string Config::info_pathname = "/etc/dcheck.conf";

Config::Config()
{
//cout << "Config::Config" << endl;
}

Config::~Config()
{
//cout << "Config::~Config" << endl;
}

void Config::set_status(Status *st)
{
status = st;
}

/********************************
 * Return length of file fname  *
 * in bytes.                    *
 * Return FALSE if it fails.    *
 ********************************/

bool Config::get_fsize(string fname, off_t *size)
{
    struct stat statbuf;

    if (stat(fname.c_str(), &statbuf) == 0)
		{
		*size = statbuf.st_size;
		return(TRUE);
		}
return(FALSE);
}



/*******************************
 * Get unavailable filesystems *
 *******************************/
string Config::get_unavailable(void)
{
Lists lists;
FS_strings_info *fs_str;
FS_control *fs_c;
list<FS_control *>::iterator i;
string str;
bool unavailable = FALSE;

for (i=lists.FS_control_list.begin(); i!=lists.FS_control_list.end(); ++i)
	{
	fs_c = (FS_control *)*i;

	fs_str = lists.get_fs_str(fs_c->name);
		
	if (!fs_str)
		{
		// This one was unavailable...
		if (!unavailable)
			{
			str = "\n# Unavailable filesystem(s)\n";
			unavailable = TRUE;
			}
		str += "FILESYSTEM = " + fs_c->name + "\nCONTROL = " + fs_c->control + "\n";

		}
	}
return str;
}

/***********************
 * Save config to disk *
 ***********************/
//
void Config::save_conf(void)
{
Lists lists;
fstream ofp;
string buffer;


list<FS_widgets_info *>::iterator i;

	
ofp.open(info_pathname.c_str(), ios::out | ios::binary | ios::trunc );
if ( ofp.is_open() )
  {
		
	buffer = (string)"# \"" + info_pathname + "\", the dcheck file\n\n";
		

	for (i=Lists::FS_widget_list.begin(); i!=Lists::FS_widget_list.end(); ++i)
		{			
		FS_widgets_info *fs = (FS_widgets_info *)*i;
			
		buffer += (string)"FILESYSTEM = " + gtk_label_get_text(GTK_LABEL(fs->name_label));
		buffer += (string)"\nCONTROL = " + 
			   ((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs->managed_check_button)))
                ? "dcheck" : "tune2fs") + "\n";

		} // End of for


		
	buffer += (string)"\nAUTHORISATION = " + ((su_auth) ? "su\n" : "sudo\n");
	buffer += (string)"STATUS_WANTED = " + ((status_wanted) ? "TRUE\n" : "FALSE\n");


// Add unavailable to buffer filesystems here
// These are in FS_control_list but not in FS_strings_list or  FS_widgets_list

	buffer += get_unavailable();
		
	ofp.write( (char *)buffer.c_str(), buffer.length() );

	if (ofp.good())
		{
		status->fix_status( "Saved configuration", TRUE);
		status->add_status_underline();
		}

	else
		{
	// Report the error
		status->fix_status("Saved configuration FAILED", TRUE);
		status->fix_status(strerror(errno), TRUE);
		status->add_status_underline();
		}

	ofp.close();
	}
else if (errno)
	{
	// Report the error 
	status->fix_status("Saved configuration FAILED - Couldn't open file", TRUE);
	status->fix_status(strerror(errno), TRUE);
	status->add_status_underline();
	}
}
//



/***************************
 * Load a file into memory *
 ***************************/
string Config::load_file(string pathname)
{
off_t size = 0; // This is a long long
ssize_t len;
int fd;
string conf_file;
char *buffer = NULL;

	
get_fsize(pathname, &size);	
if (!size) return ("");

if ( (buffer = (char *)calloc(size + 1, 1)) ) /* include zero byte */
	{
	/* Open the file and load it into buffer */
	fd = open(pathname.c_str(), O_RDONLY);
	if (fd >= 0)
		{
		len = read(fd, buffer, size);
		if (len == size)
			{
			conf_file = buffer;
			close(fd);
			free(buffer);
			return conf_file;
			}
		}
	}
if (buffer) free(buffer);
return ("");
}





char *Config::get_right(char *c, char *old_end)
{
char *end;
gboolean in_quote = FALSE;

/* Remove leading white space */
while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */

/* Skip an assignment character */
if ((*c == '=') || (*c == ':'))
	{
	c++;

	/* Skip white space after assignment character */
	while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */
    }

/* Now look for end of line starting here, or start of a comment */
end = c;

/*
 Find the end, comments are # or ; (Assumes no white space in the value and line ends with newline)
If old_end is defined, the character is the old character at the end which is replaced by the zero
*/
	
while ( (*end) && ( (in_quote && (*end != '\n')) ||
		(!in_quote && (*end != ' ') && (*end != '\t') && (*end != '#') && (*end != ';') && (*end != '\n') ) ) )
	{
	if (*end == '\"')
		{
		if (in_quote && (*(end+1) == '\"')) end++;  /* literal */
		else in_quote ^= 1;
		}
	end++;
	}
  
if (old_end) *old_end = *end;  /* The character that was replaced by a zero */
*end = 0;

return(c);
}





void Config::load_conf(void)
{
Lists lists;
string name, control;

char *c;   // current position
char end;
	

	
string config_file = load_file(info_pathname);

//cout << "dcheck_main: Config::load_conf - config_file = \n" << config_file << endl;

su_auth       = TRUE;	// default
status_wanted = TRUE; // default


//cout << "ENTERED Config::load_conf: config_file=\n" << config_file << endl;

if (!config_file.empty())
  {
  // Now parse it
	c = (char *)config_file.c_str();

	while (c && *c)
		{
		while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */

    if ( (*c == '#') || (*c == ';') || (*c == '\n') )
			{
			/* comments, newlines */
//cout << "Config::load_conf - FOUND COMMENT or NEWLINE\n";

			while (*c && *c != '\n') c++;
			if (*c) c++;
			}

		else if ( (strncasecmp(c, "FILESYSTEM", 10)==0) && ( (*(c+11)==' ') || (*(c+11)=='=')) )
			{
      name.clear();
			control.clear();


			c += 10;
	  	c = get_right(c, &end);

			if (*c)
				{
        name = c;
        }
			c += strlen(c);
			*c = end;
          

//cout << "Config::load_conf - FOUND FILESYSTEM name=" << name << endl;

      if (name.length())
        {
				control = "tune2fs"; // default

				if (c && *c)
					{
					while ( (*c == ' ') || (*c == '\t') ) c++;   /* white space */

  				if ( (*c == '#') || (*c == ';') || (*c == '\n') )
						{
						// comments, newlines
	//					cout << "Config::load_conf - FOUND COMMENT or NEWLINE 2\n";

						while (*c && *c != '\n') c++;
						if (*c) c++;
						}

					if ( (strncasecmp(c, "CONTROL", 7)==0) && ( (*(c+8)==' ') || (*(c+8)=='=')) )
  					{        
  					c += 7;
  					c = get_right(c, &end);
 
						if (*c)
							{
      				control = c;

 	//						cout << "Config::load_conf - FOUND CONTROL "<< control << endl;
     					}
						c += strlen(c);
						*c = end;
						}
					}



//			cout << "Config::load_conf - LISTING name= " << name <<" control= " << control << endl;

				lists.add_to_control_list(name, control);
				}

			} 

		else if ( (strncasecmp(c, "AUTHORISATION", 13)==0) && ( (*(c+14)==' ') || (*(c+14)=='=')) )
      {
	//		cout << "Config::load_conf - FOUND AUTHORISATION\n";
      c += 13;
      c = get_right(c, &end);
      if (*c)
        {
        if ( strcasecmp(c, "sudo")==0 ) su_auth = FALSE;
			  c += strlen(c);
			  *c = end;
        }
      }

		else if ( (strncasecmp(c, "STATUS_WANTED", 13)==0) && ( (*(c+14)==' ') || (*(c+14)=='=')) )
      {
			//cout << "Config::load_conf - FOUND STATUS_WANTED\n";
      c += 13;
      c = get_right(c, &end);
      if (*c)
        {
        status_wanted = ( (strcasecmp(c, "TRUE")==0) || (strcasecmp(c, "YES")==0)|| (strcasecmp(c, "1")==0)) ? TRUE : FALSE;				
			  c += strlen(c);
			  *c = end;
        }
      }

    else
			{
			/* UNRECOGNIZED, ignore to eol */
 			cout << "dcheck_main: Config::load_conf - UNRECOGNIZED " << c << endl;
				
			while (*c && (*c != '\n')) c++;
			}

//		cout <<"Config::load_conf: end of loop, c= " << c << endl;

		} // END OF while (c && *c)
  }
//cout <<"Config::load_conf: Leaving" << endl;

return;
}



/*
 * Get formatted file size from data
 */
string Config::get_fs_size(string data)
{
char *p;

// size - for this we need "Block count:" and "Block size:"
if ( (p = strstr( (char *)data.c_str(), "Block count:") ) )
  {
  unsigned long long block_count= 0;
  unsigned long long block_size = 0;
  double size;
  char *endptr;


  p += 12;
  block_size = strtold(p, &endptr);

  if ( (p = strstr((char *)data.c_str(), "Block size:")) )
    {
    p += 11;
    block_count = strtold(p, &endptr);


    size = (double)(block_count * block_size); // Bytes

    if (size > 0)
      {
      if (size < 1024)
        {
        // Bytes
        return misc::convert_f_to_string(size) + (string)" Bytes";
        }
      
      size /= 1024;
      if (size < 1024)
        {
        // KiB
				return misc::convert_f_to_string(size) + (string)" KiB";
        }
      size /= 1024;

      if (size < 1024)
        {
        // MiB
				return misc::convert_f_to_string(size) + (string)" MiB";
        }
      size /= 1024;
      if (size < 1024)
        {
        // GiB
				return misc::convert_f_to_string(size) + (string)" GiB";
        }
      size /= 1024;
      // TiB
			return misc::convert_f_to_string(size) + (string)" TiB";
      }
    }
  }
return (string)"<UNKNOWN>";
}





void Config::fix_dcheck(void)
{
Lists lists;
FS_strings_info *fs_str;
FS_widgets_info *fs_w;
string result;
string command;

list<FS_strings_info *>::iterator i;

status->fix_status((string)"Config::fix_dcheck - ENTERED", TRUE);

// Iterate through the filesystems (assumes FS_string_list has been initialised)
//  and fix those that are under dcheck control

	
for (i=lists.FS_string_list.begin(); i!=lists.FS_string_list.end(); ++i)
  {
  fs_str = (FS_strings_info *)*i;

  if (fs_str->control == "dcheck")
    {
    // GOT ONE, so fix it
    // create the command for this filesystem
//    snprintf(command, 1024, "tune2fs -c -1 %s", (char *)fs_str->name);     // filesystem name      
		command = (string)"tune2fs -C 1 -c -1 " + (string)fs_str->name;
		status->fix_status( (string)"Config::fix_dcheck - command=:-\n" + (string)command + (string)"\n", FALSE);

		result = misc::get_output(command);
			
		status->fix_status( (string)"Config::fix_dcheck - result=:-\n" + (string)result + (string)"\n", FALSE);

			
    if (strstr((char *)result.c_str(), "Setting maximal mount count to -1"))
      {
      // Change it in list
			fs_str->num_mounts = "1"; //arbtrary value
      fs_str->max_mounts = "-1";

// This need to change the widget list as well
			fs_w = lists.get_fs_w(fs_str);
			gtk_entry_set_text(GTK_ENTRY(fs_w->num_mounts_entry), "1");
			gtk_entry_set_text(GTK_ENTRY(fs_w->max_mounts_entry), "-1");
			}
    }
  }
}


