/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       dcheck.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       04-Mar-2013 17.19.28 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* The main window                                                           *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "dcheck.h"

using namespace misc;

GtkWidget *Dcheck::window             = NULL;

GdkPixbuf *Dcheck::dtime_pixbuf       = NULL;
GtkWidget *Dcheck::dtime_image        = NULL;

GdkPixbuf *Dcheck::dcheck_icon_pixbuf = NULL;
GtkWidget *Dcheck::dcheck_icon_image  = NULL;

GdkPixbuf *Dcheck::down_pixbuf        = NULL;

GtkWidget *Dcheck::apply_button       = NULL;
GtkWidget *Dcheck::quit_button        = NULL;
GtkWidget *Dcheck::change_last_check_button = NULL;

GtkWidget *Dcheck::menu               = NULL;
GtkWidget *Dcheck::menu_left          = NULL;

About *Dcheck::about                  = NULL;
Last_check *Dcheck::last_check        = NULL;



// For pop-up menu
char *Dcheck::action_txt[6] =
  {
  (char *)"Apply",
  (char *)"Change Last Check",
  (char *)"Change Authorisation to \"sudo\"",
  (char *)"Disable status window",
  (char *)"About",
  (char *)"Quit"
  };

char *Dcheck::interval_type_txt[4] =
  {
  (char *)"<none>",
  (char *)"day(s)",
  (char *)"week(s)",
  (char *)"month(s)",
  };

GtkWidget *Dcheck::interval_type_widget = NULL; // for callbacks of left popup menu




Dcheck::Dcheck(void)
{
//cout << "Dcheck::Dcheck" << endl;
}

Dcheck::~Dcheck(void)
{
//cout << "Dcheck::~Dcheck" << endl;


if (dcheck_icon_pixbuf)
	{
	g_object_unref(G_OBJECT(dcheck_icon_pixbuf));
	dcheck_icon_pixbuf = NULL;
	}
if (dtime_pixbuf)
	{
	g_object_unref(G_OBJECT(dtime_pixbuf));
	dtime_pixbuf = NULL;
	}

gtk_main_quit();


//cout << "Dcheck::~Dcheck - LEAVING" << endl; 
cout << "Goodbye World" << endl;
exit(EXIT_SUCCESS); // prevent it from coming back!
}

void Dcheck::callback_destroy(GtkWidget *widget, Dcheck *dc)
{
		
//cout << "Dcheck::callback_destroy" << endl;
delete dc;
}

// It comes here twice!

void Dcheck::callback_quit(GtkWidget *widget, Dcheck *dc)
{
	
//cout << "Dcheck::callback_quit" << endl;
	
if (dc->about)
	{
	//cout << "Dcheck::callback_quit - GOT dc->about" << endl;

	dc->about->callback_about_close(NULL,  about);
	}
if (dc->last_check)
	{
	////cout << "Dcheck::callback_quit - GOT dc->last_check" << endl;

	dc->last_check->callback_last_check_cancel(NULL, last_check);
	}

//cout << "Dcheck::callback_quit - DO gtk_widget_destroy" << endl;
gtk_widget_destroy(dc->window);
	
//cout << "Dcheck::callback_quit - LEAVING" << endl;
}





void Dcheck::callback_change_last_check(GtkWidget *widget, Dcheck *dc)
{
//cout << "Dcheck::callback_change_last_check" << endl;



if (!dc->last_check)
	{
	dc->last_check = new(Last_check);
		
//cout << "Dcheck::callback_change_last_check - DONE new(Last_check)" << endl;

	dc->last_check->change_last_check_gui(NULL, dc, &dc->status);
	}
//else cout << "Dcheck::callback_change_last_check - Last_check aready running" << endl;
}


void Dcheck::callback_apply(GtkWidget *widget, Dcheck *dc)
{
Lists lists;
FS_widgets_info *fs;


string num_mounts;    // Values after applying
string max_mounts;
string type;
string interval;
int interval_type;
struct tm * tm_now;
time_t now;
char *t;
	
list<FS_widgets_info *>::iterator i;

// Values befor applying    
string size_txt_1, state_txt_1, mount_point_txt_1,
      last_check_txt_1, num_mounts_txt_1,
      max_mounts_txt_1, interval_txt_1, interval_type_txt_1;
	


// Values after applying    
string size_txt, state_txt, mount_point_txt,
      last_check_txt, num_mounts_txt,
      max_mounts_txt, interval_txt, interval_type_txt;
	

string text;
string command;
GtkEntryBuffer *entry_buffer;

//cout << "Dcheck::callback_ok - ENTERED:" << endl;;


if  ( ( (now	= time(NULL)) != -1 )   && 
      ( (tm_now = localtime(&now)) )  && 
   		( ( t = asctime(tm_now) ) ) )
	{
	dc->status.fix_status((string)t, TRUE);
	}

for (i=lists.FS_widget_list.begin(); i!=lists.FS_widget_list.end(); ++i)

	{
			
	FS_widgets_info *fs = (FS_widgets_info *)*i;


// GET EXISTNG VALUES TO SEE WHAT HAS CHANGED AFTER APPLYING

// NOTE: get_tune gets everything so that it can be compared later...
  dc->get_tune( (string)gtk_label_get_text(GTK_LABEL(fs->name_label)),
           size_txt_1, state_txt_1,
           mount_point_txt_1, last_check_txt_1,
           num_mounts_txt_1, max_mounts_txt_1,
           interval_txt_1, interval_type_txt_1, NULL);




		

// GATHER  INFO FROM WIDGETS

  interval = (gchar *)gtk_entry_get_text(GTK_ENTRY(fs->interval_entry));
		
	string type_txt = gtk_button_get_label(GTK_BUTTON(fs->interval_type_button));
		
	if (type_txt == "<none>")        type = "";
	else if (type_txt == "day(s)")   type =  "d";
	else if (type_txt == "week(s)")  type =  "w";
	else if (type_txt == "month(s)") type =  "m";
	else  type = "";


  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs->managed_check_button))) // if managed
    {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs->dcheck_button)))
      {
      num_mounts = "9999"; // Check next mount, num_mounts higher than max_mounts
      max_mounts = "9998";
      }
    else
      {
      num_mounts = gtk_entry_get_text(GTK_ENTRY(fs->num_mounts_entry));
			if (atoi(num_mounts.c_str()) >= 9998)
				{
				num_mounts = "10";
				gtk_entry_set_text(GTK_ENTRY(fs->num_mounts_entry), num_mounts.c_str()); // Arbitrary value;
				}
      max_mounts = "-1";  // Don't check
      }
    }
  else // Not managed, so get the values
    {
    max_mounts = gtk_entry_get_text(GTK_ENTRY(fs->max_mounts_entry));

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs->dcheck_button))) // If we want a check
      {
      // Disk check wanted next mount but not managed_check_button
      // Set num_mounts to 1 more than max_mounts making sure that max_mounts is > 0
      // If max_mounts <= 0, set it to say 10.
//      gchar buffer[1024];
      int mm = atoi( max_mounts.c_str() );

      if (mm <= 0)
				{
				max_mounts = "10"; // arbitrary value...
				mm = 10;
				}

      // Convert (mm+1) to text for num_mounts   -   Make num_mounts 1 larger than max_mounts
			num_mounts = convert_i2str(mm+1, 0);
			
      }
    else
      {
      num_mounts = gtk_entry_get_text(GTK_ENTRY(fs->num_mounts_entry));
      }
    }

// APPLY WITH tune2fs

	// -C num_mounts
	// -c max_mounts
	// -i interval
	// d,w or m
	// filesystem name
    

	command = (string)"tune2fs -C " + num_mounts + " -c " + max_mounts + 
			" -i " + interval + type + " " + gtk_label_get_text(GTK_LABEL(fs->name_label));



//cout << "***callback_ok - command=\n  " << command << endl;

		
//	result = get_output(command);  // Apply it with tune2fs - we don't actually need the result

get_output(command);  // Apply it with tune2fs - we don't actually need the result

// Now change the widgets to reflect the new values which just might be different!
// e.g. if it failed to actually update for some reason - can't seee why not though.


// GATHER THE INFO THAT tune2fs ACTUALLY DID
// AND PUT IT INTO THE WIDGETS


// NOTE: get_tune gets everything so that could be used...
  dc->get_tune( (string)gtk_label_get_text(GTK_LABEL(fs->name_label)),
           size_txt, state_txt,
           mount_point_txt, last_check_txt,
           num_mounts_txt, max_mounts_txt,
           interval_txt, interval_type_txt, NULL);


// size should be unchanged! Fix anyway...
  gtk_label_set_text(GTK_LABEL(fs->size_label), size_txt.c_str());

// state
  gtk_label_set_text(GTK_LABEL(fs->state_label), state_txt.c_str());


// Check tooltip
  text = "Check\nTick for filesystem check at next mount\n" + mount_point_txt + "\nLast checked:\t\t" + last_check_txt;
  gtk_widget_set_tooltip_text(fs->dcheck_button, text.c_str());



// gtk_entry_set_text doesn't seem to work properly
// num_mounts_txt is correct but display is the OLD VALUE if it was changed with Check
// Try using its buffer instead... This works...
    
// num_mounts
  entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs->num_mounts_entry));
  gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
  gtk_entry_buffer_insert_text(entry_buffer, 0, num_mounts_txt.c_str(), -1); // insert text
 
// max_mounts
  entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs->max_mounts_entry));
  gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
  gtk_entry_buffer_insert_text(entry_buffer, 0, max_mounts_txt.c_str(), -1); // insert text
    
// interval
  entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs->interval_entry));
  gtk_entry_buffer_delete_text(entry_buffer, 0, -1);               // delete all the characters in the buffer
  gtk_entry_buffer_insert_text(entry_buffer, 0, interval_txt.c_str(), -1); // insert text

// interval type
// cout << "Dcheck::callback_ok - interval_type_txt= " << interval_type_txt << endl;
		
	if (interval_type_txt == "<none>") interval_type = 0;
	else if ((interval_type_txt == "day") || (interval_type_txt == "days")) interval_type = 1;
	else if ((interval_type_txt == "week") || (interval_type_txt == "weeks")) interval_type = 2;
	else if ((interval_type_txt == "month") || (interval_type_txt == "months")) interval_type = 3;
	else interval_type = 0;

	gtk_button_set_label(GTK_BUTTON(fs->interval_type_button), dc->interval_type_txt[interval_type]);

// CHECK OUT WHAT HAS CHANGED
// CORRECT THE STRING LIST
// NOTIFY USER IN STATUS WINDOW

  dc->status.fix_status((string)"Dcheck: Main Window - " + gtk_label_get_text(GTK_LABEL(fs->name_label)), TRUE) ;
  dc->status.fix_status((string)("Executed command: " + command), TRUE);
	bool changed = FALSE;
		
	FS_strings_info *fs_str = lists.get_fs_str((string)gtk_label_get_text(GTK_LABEL(fs->name_label)));

	if (size_txt_1 != size_txt)
		{
		// size has changed - surely that's impossible!
		dc->status.fix_status((string)("SIZE changed! from " + fs_str->size + " to " + size_txt), FALSE);
		fs_str->size = size_txt;
		changed = TRUE;
		}



	if (state_txt_1 != state_txt)
		{
		// state has changed - I suppose it's possible
		dc->status.fix_status((string)("STATE changed! from " + fs_str->state + " to " + state_txt), FALSE);
		fs_str->state = state_txt;
		changed = TRUE;
		}
		

	string control_txt = (string)((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs->managed_check_button))) ?  "dcheck" : "tune2fs");

	if (fs_str->control != control_txt)
		{
		// managed has changed
		dc->status.fix_status((string)("MANAGED changed from " + fs_str->control + " to " + control_txt), FALSE);
		fs_str->control   = control_txt;
		changed = TRUE;
		}

// Check at Next Mount--------------------------------------
	bool check_next_mount = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs->dcheck_button));
	if (fs_str->check_wanted != check_next_mount)
		{
		// check has changed
		dc->status.fix_status( (string)("CHECK NEXT MOUNT changed from " +
		                       (string)( (fs_str->check_wanted) ? "wanted" : "not wanted") + 
		                       " to " + 
		                       (string)( (check_next_mount) ?  "wanted" : "not wanted") ), FALSE);
		fs_str->check_wanted  = check_next_mount;
		changed = TRUE;
		}

	    
	if (mount_point_txt_1 != mount_point_txt)
		{
		// mount_point has changed - Oh Dear!
		dc->status.fix_status( (string)"MOUNT POINT changed! from " + fs_str->mount_point + " to " + mount_point_txt, FALSE);
		fs_str->mount_point = mount_point_txt;
		changed = TRUE;
		}


	if (num_mounts_txt_1 != num_mounts_txt)
		{
		// num_mounts has changed
		dc->status.fix_status( (string)"NUM MOUNTS changed from " + fs_str->num_mounts + " to " + num_mounts_txt, FALSE);
		fs_str->num_mounts = num_mounts_txt;
		changed = TRUE;		}
	    

	if (max_mounts_txt_1 != max_mounts_txt)
		{
		// max_mounts has changed
		dc->status.fix_status( (string)"MAX MOUNTS changed from " + max_mounts_txt_1 + " to " + max_mounts_txt, FALSE);
		fs_str->max_mounts = max_mounts_txt;		
		changed = TRUE;		}
	    

	if ((interval_txt_1 != interval_txt) || (interval_type_txt_1 != interval_type_txt))
		{
		// interval has changed
		dc->status.fix_status( (string)"INTERVAL changed from " +
				((interval_txt_1=="0") ? "<none>" : (interval_txt_1 + " " + interval_type_txt_1)) +
				" to " + ((interval_txt=="0") ? "<none>" : (interval_txt + " " + interval_type_txt)), FALSE);
			
		fs_str->interval = interval_txt;
		fs_str->interval_type = interval_type_txt;
		changed = TRUE;	
		}
	    
    
	if (!changed)
		{
		dc->status.fix_status( (string)"NO CHANGE", FALSE);
		}



	} // End of for (i=Lists::FS_widget_list.begin(); ...

dc->config.save_conf();
}

/********************
 * The About window *
 ********************/
void Dcheck::callback_about(GtkWidget *widget, Dcheck *dc)
{
cout << "Dcheck::callback_about" << endl;
if (!dc->about)
	{
	dc->about = new(About);
	dc->about->about_gui(NULL, dc);
	}
//else cout << "Dcheck::callback_about - About is already running!" << endl;
}



  


void Dcheck::callback_change_auth(GtkWidget *widget, Dcheck *dc)
{
//cout << "Dcheck::callback_change_auth" << endl;

dc->config.su_auth = (!dc->config.su_auth);
   
gtk_widget_destroy(dc->menu);
dc->create_menu();
dc->status.fix_status( ((dc->config.su_auth) ? "Changed authorisation to \"su\"" : "Changed authorisation to \"sudo\""), TRUE);
dc->config.save_conf();
}

void Dcheck::callback_change_status_window(GtkWidget *widget,  Dcheck *dc)
{
//cout << "Dcheck::callback_change_status_window" << endl;

dc->config.status_wanted = dc->status.status_wanted = (!dc->config.status_wanted);

// If the status window is open, it should be closed...
if (!dc->config.status_wanted && dc->status.status_running())
	{
	dc->status.callback_status_close(NULL, NULL);
	}

gtk_widget_destroy(menu);
dc->create_menu();
if (dc->status.status_wanted) dc->status.fix_status("Enabled status window", TRUE);
dc->config.save_conf();
}



/*************************************
 * A "Managed" check box was clicked *
 *************************************/

void Dcheck::callback_managed_clicked(GtkWidget *widget, Dcheck *dc)
{
Lists lists;

//cout << "Dcheck::callback_managed_clicked - ENTERED" << endl;

// Get everything about this filesystem
FS_widgets_info *fs_w = lists.get_fs_w_of_widget(GTK_WIDGET(widget));
FS_strings_info *fs_str = lists.get_fs_str(fs_w); // ? change current values


if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
	{
	// if selected - collect old values and insert new vales
	fs_w->old_managed_num_mounts = (string)gtk_entry_get_text(GTK_ENTRY(fs_w->num_mounts_entry));
	fs_w->old_managed_max_mounts = (string)gtk_entry_get_text(GTK_ENTRY(fs_w->max_mounts_entry));

	gtk_entry_set_text(GTK_ENTRY(fs_w->max_mounts_entry), "-1"); // num_mounts unchanged
		
	}
else
	{
	// if de-selected put back the old values

	gtk_entry_set_text(GTK_ENTRY(fs_w->max_mounts_entry), fs_w->old_managed_max_mounts.c_str());

	//probably unnecessary
	gtk_entry_set_text(GTK_ENTRY(fs_w->num_mounts_entry), fs_w->old_managed_num_mounts.c_str());
	}
}

/***********************************
 * A "Check" check box was clicked *
 ***********************************/

void Dcheck::callback_dcheck_clicked(GtkWidget *widget, Dcheck *dc)
{
Lists lists;
GtkEntryBuffer *entry_buffer;
string num_mounts, max_mounts;
	
cout << "Dcheck::callback_check_clicked - ENTERED" << endl;

// Get everything about this filesystem
FS_widgets_info *fs_w   = lists.get_fs_w_of_widget(GTK_WIDGET(widget));
FS_strings_info *fs_str = lists.get_fs_str(fs_w); // ? change current values

if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) // if want check next mount
	{
	// FIRST store old values
	fs_w->old_dcheck_num_mounts = (string)gtk_entry_get_text(GTK_ENTRY(fs_w->num_mounts_entry));
	fs_w->old_dcheck_max_mounts = (string)gtk_entry_get_text(GTK_ENTRY(fs_w->max_mounts_entry));

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs_w->managed_check_button))) // if managed
		{
		num_mounts = "9999";   // Check next mount, num_mounts higher than max_mounts
    max_mounts = "9998";
		}
	else // if not managed
		{
//cout << "Dcheck::callback_check_clicked - not managed" << endl;
		
		int mm = atoi( (max_mounts = fs_w->old_dcheck_max_mounts).c_str() );

		if (mm <= 0)  // Otherwise max_mounts is unchanged
			{
			max_mounts = "10"; // arbitrary value...
			mm = 10;
			}
			
//cout << "Dcheck::callback_check_clicked -1 not managed mm= " << mm << endl;
			
      // Convert (mm+1) to text for num_mounts   -   Make num_mounts 1 larger than max_mounts
			num_mounts = convert_i2str( mm+1, 0 );

		}


cout << "Dcheck::callback_check_clicked - num_mounts= " << num_mounts << " : max_mounts= " << max_mounts << endl;

	// Insert new values
	entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_w->num_mounts_entry));
  gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
  gtk_entry_buffer_insert_text(entry_buffer, 0, num_mounts.c_str(), -1); // insert text

//	gtk_entry_set_text(GTK_ENTRY(fs_w->num_mounts_entry), num_mounts.c_str());


	entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_w->max_mounts_entry));
  gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
  gtk_entry_buffer_insert_text(entry_buffer, 0,max_mounts.c_str(), -1); // insert text

		
//	gtk_entry_set_text(GTK_ENTRY(fs_w->max_mounts_entry), max_mounts.c_str());
	
	}
else	// Put back old values
	{
	gtk_entry_set_text(GTK_ENTRY(fs_w->num_mounts_entry), fs_w->old_dcheck_num_mounts.c_str());
	gtk_entry_set_text(GTK_ENTRY(fs_w->max_mounts_entry), fs_w->old_dcheck_max_mounts.c_str());
	}
}


void Dcheck::callback_insert_text_handler(GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
Lists lists;
GtkEditable *editable = GTK_EDITABLE(entry);
int i=0, count=0;
int min, max, val;
string val_txt;
GtkEntryBuffer *entry_buffer;
string start_text;                  // a copy of the text already in the buffer

string result;
	
FS_widgets_info *fs_w = lists.get_fs_w_of_widget(GTK_WIDGET(entry));

if (data == (gpointer)0)
  {
  min = 0;
  max = 9999;
  }
else if (data == (gpointer)1)
  {
  min = -1;
  max = 9999;
  }
else
  {
  min = 0;
  max = 9999;
  }
   
  
entry_buffer = gtk_entry_get_buffer(entry);

start_text = gtk_entry_buffer_get_text(entry_buffer); // This is always empty when only one character is in the buffer- it should be the old value

//cout << "Dcheck::callback_insert_text_handler - start_text= \"" << start_text << "\" : size (bytes)="  <<
//		gtk_entry_buffer_get_bytes(entry_buffer) << endl;


//cout << "Dcheck::callback_insert_text_handler - text= \"" << text << " \"" << endl;

	
// We need to check that all chars in text are digits
// if 1st char of text is '-' and position is 0, that is OK.

if ((text[i] == '-') && (*position == 0))
	{
	result[count++] = text[i++];
	}

for(; i < length; i++)
	{
	if (!isdigit(text[i])) continue;
	result[count++] = text[i++];
	}
//result[count] = 0;
  
if (count > 0)
  {
  g_signal_handlers_block_by_func (G_OBJECT (editable),
                                     (gpointer)callback_insert_text_handler,
                                     data);
  gtk_editable_insert_text (editable, result.c_str(), count, position);
  g_signal_handlers_unblock_by_func (G_OBJECT (editable),
                                       (gpointer)callback_insert_text_handler,
                                       data);
  }
g_signal_stop_emission_by_name (G_OBJECT (editable), "insert_text");

val_txt = (string)gtk_entry_buffer_get_text(entry_buffer);
val = atoi(val_txt.c_str());

if ((val < min) || (val > max)) // if out of range, restore the original contents
  {
  gtk_entry_buffer_delete_text(entry_buffer, 0, -1); // delete all the characters in the buffer
  gtk_entry_buffer_insert_text(entry_buffer, 0, start_text.c_str(), -1); //re-insert original text
	return;
	}
  
if ( fs_w && (fs_w->interval_entry==GTK_WIDGET(entry)) )
	{
	// Get the FS_strings_info
	FS_strings_info *fs_str = lists.get_fs_str(fs_w);
	string start_interval = fs_str->interval;
//
//	cout << "Dcheck::callback_insert_text_handler - GOT fs 1:\n  val=" << val  << endl;
//			<< " : start_interval=" << start_interval
//			<< " : get_active= " << (gtk_combo_box_get_active(GTK_COMBO_BOX(fs_w->interval_type_button))) << endl;
//		




//		if ((val > 0) && (atoi(start_interval.c_str())==0) && (gtk_combo_box_get_active(GTK_COMBO_BOX(fs_w->interval_type_button)))==0)  // if it is now >0 and was previously 0
	if ((val > 0) && (atoi(start_interval.c_str())==0) && ( (string)gtk_button_get_label(GTK_BUTTON(fs_w->interval_type_button)) == "<none>"   ))  // if it is now >0 and was previously 0
   {
//		cout << "Dcheck::callback_insert_text_handler - GOT fs 2" << endl;
    // Set the type to default of days
// 		gtk_combo_box_set_active(GTK_COMBO_BOX(fs_w->interval_type_button),  1);
		gtk_button_set_label(GTK_BUTTON(fs_w->interval_type_button),interval_type_txt[1]);

		g_signal_handlers_block_by_func (G_OBJECT (editable),
                                     (gpointer)callback_insert_text_handler,
                                     data);
		gtk_editable_delete_text(editable, 0, -1); // clear the buffer
		i=0;
		gtk_editable_insert_text (editable, val_txt.c_str(), val_txt.length(), &i);

		g_signal_handlers_unblock_by_func (G_OBJECT (editable),
                                      (gpointer)callback_insert_text_handler,  data);
                                   
		fs_str->interval = val_txt;
		fs_str->interval_type = "days";
//		cout << "Dcheck::callback_insert_text_handler - DONE interval=" << fs_str->interval << endl;
		
    }
		
  else if (val==0)
    {
//		cout << "Dcheck::callback_insert_text_handler - GOT fs 3" << endl;
//    gtk_combo_box_set_active(GTK_COMBO_BOX(fs_w->interval_type_button),  0);
		
		gtk_button_set_label(GTK_BUTTON(fs_w->interval_type_button),interval_type_txt[0]);

		fs_str->interval = "0";
		fs_str->interval_type = "<none>";
//		cout << "Dcheck::callback_insert_text_handler - DONE interval=" << fs_str->interval << endl;
    }
  }

}

//======================================================================================
/************************************ LEFT MENU ***************************************/
//======================================================================================


 
void Dcheck::interval_type_changed(FS_widgets_info *fs_w,
                                   string old_type, int new_type, Dcheck*dc)
{
Lists lists;

//cout << "Dcheck::interval_type_changed - ENTERED" << endl;

if (fs_w)
	{
	FS_strings_info *fs_str = lists.get_fs_str(fs_w);	
	gtk_button_set_label(GTK_BUTTON(fs_w->interval_type_button),interval_type_txt[new_type]);

	if (old_type == "none>") // originally type was <none>
		{
		
//		cout << "Dcheck::interval_type_changed - old_type= " << old_type << endl;
			
	// It has changed so type is no longer <none>
		if ( (string)gtk_entry_get_text(GTK_ENTRY(fs_w->interval_entry)) == "0")
			{
//			cout << "Dcheck::interval_type_changed - fs_w->interval_entry= " << "0" << endl;
				
//cout << "Dcheck::interval_type_changed -  current " << gtk_entry_get_text(GTK_ENTRY(fs_w->interval_entry)) << endl;
			gtk_entry_set_text(GTK_ENTRY(fs_w->interval_entry), "1");
			fs_str->interval = 1;
			fs_str->interval_type	= "days";
			return;
			}
		}


	if (new_type == 0)   // if type is now <none>
		{
		gtk_entry_set_text(GTK_ENTRY(fs_w->interval_entry), "0");
 // 		cout << "Dcheck::interval_type_changed - INTERVAL SET to 0" << endl;

		// Fix fs_str->interval and type
		fs_str->interval = "0";
		fs_str->interval_type = "<none>";				
		}
	else                                                         // if type is not <none>
		{
		int val = atoi(fs_str->interval.c_str());                  // get interval
		if ((fs_str->interval_type == "<none>") && (val == 0))     // if start type is <none> and no interval
			{
			gtk_entry_set_text(GTK_ENTRY(fs_w->interval_entry), "1");// make interval 1, type stays the same
			fs_str->interval_type	= "days";
			}
		else
			{
			// Fix fs_str->interval and type
			fs_str->interval_type = interval_type_txt[new_type];			
			fs_str->interval = gtk_entry_get_text(GTK_ENTRY(fs_w->interval_entry));

			}
		}
	}
}


void Dcheck::callback_none(GtkWidget *widget, Dcheck*dc)

{
Lists lists;
	
FS_widgets_info *fs_w = lists.get_fs_w_of_widget(dc->interval_type_widget);

string interval_type_txt = (string)gtk_button_get_label(GTK_BUTTON(fs_w->interval_type_button));
	
//cout << "Dcheck::callback_none - ENTERED 4- "<< endl;
//cout <<          "interval_type_txt=" << interval_type_txt << endl;

// Not sure whether the widget is actually passed through - it's not!

if (interval_type_txt != "<none>")
	{
	dc->interval_type_changed(fs_w, interval_type_txt, 0, dc);
	}

}


void Dcheck::callback_days(GtkWidget *widget, Dcheck*dc)

{
//cout << "callback_days - ENTERED " << endl;
Lists lists;
	
FS_widgets_info *fs_w = lists.get_fs_w_of_widget(dc->interval_type_widget);
string interval_type_txt = (string)gtk_button_get_label(GTK_BUTTON(fs_w->interval_type_button));


if (interval_type_txt != "day(s)")
	{
	dc->interval_type_changed(fs_w, interval_type_txt, 1, dc);
	}
}


void Dcheck::callback_weeks(GtkWidget *widget, Dcheck*dc)

{
//cout << "callback_weeks - ENTERED " << endl;
Lists lists;
	
FS_widgets_info *fs_w = lists.get_fs_w_of_widget(dc->interval_type_widget);
string interval_type_txt = (string)gtk_button_get_label(GTK_BUTTON(fs_w->interval_type_button));


if (interval_type_txt != "week(s)")
	{
	dc->interval_type_changed(fs_w, interval_type_txt, 2, dc);
	}

}



void Dcheck::callback_months(GtkWidget *widget, Dcheck*dc)

{
//cout << "callback_months - ENTERED " << endl;
Lists lists;
	
FS_widgets_info *fs_w = lists.get_fs_w_of_widget(dc->interval_type_widget);
string interval_type_txt = (string)gtk_button_get_label(GTK_BUTTON(fs_w->interval_type_button));

if (interval_type_txt != "month(s)")
	{
	dc->interval_type_changed(fs_w, interval_type_txt, 3, dc);
	}


}
//======================================================================================
/******************************** END LEFT MENU ***************************************/
//======================================================================================





/*
 * Get formatted file size
*/
string Dcheck::get_fs_size(string data)
{
char *p;

// size - for this we need "Block count:" and "Block size:"
if ( (p = strstr((char *)data.c_str(), "Block count:")) )
  {
  unsigned long long block_count= 0;
  unsigned long long block_size = 0;
  double size;
  char *endptr;


    p += 12;
  block_size = strtold(p, &endptr);

  if ( (p = strstr((char *)data.c_str(), "Block size:")) )
    {
    p += 11;
    block_count = strtold(p, &endptr);


    size = (double)(block_count * block_size); // Bytes

    if (size > 0)
      {
      if (size < 1024)
        {
        // Bytes
 				return (convert_f_to_string(size) + (string)" Bytes");
        }
      
      size /= 1024;
      if (size < 1024)
        {
        // KiB
        return (convert_f_to_string(size) + (string)" KiB");
        }
      size /= 1024;

      if (size < 1024)
        {
        // MiB
        return (convert_f_to_string(size) + (string)" MiB");
        }
      size /= 1024;
      if (size < 1024)
        {
        // GiB
        return (convert_f_to_string(size) + (string)" GiB");
        }
      size /= 1024;
      // TiB
      return (convert_f_to_string(size) + (string)" TiB");
      }
    }
  }
return "<UNKNOWN>";
}




/******************************************************
 * Returns the file system label name or "" in memstr *
 ******************************************************/
string Dcheck::get_label_name(string data)
{
char *p, *t;
char c;
string label_name;

p = strstr( (char *)data.c_str(), "Filesystem volume name:");
if (p)
  {
  p += 23;
  while ((*p==' ')||(*p=='\t')) p++;
  if (STRIEQN(p, "<none>", 6))
    {
    label_name.clear();
    }
  else
    {
    t = p;
    while ((*t)&&((*t)!='\n')) t++;
    c = *t;
    *t = 0;

    label_name = (string)"LABEL=" + p;

    *t = c;
    }
  }
else label_name.clear();
return label_name;
}





/*****************************************
 * Returns the uuid name or "" in memstr *
 *****************************************/ 
string Dcheck::get_uuid_name(string data)
{
char *p, *t;
char c;
string uuid_name;

p = strstr( (char *)data.c_str(), "Filesystem UUID:");
if (p)
  {
  p += 16;
  while ((*p==' ')||(*p=='\t')) p++;
  t = p;
  while ((*t)&&((*t)!='\n')) t++;
  c = *t;
  *t = 0;

  uuid_name = (string)"UUID=" + p;

  *t = c;
  }

else uuid_name.clear();
return uuid_name;
}




// We actually want to add the shortest name for this filesystem
// This will usually be LABEL=XXXX or /dev/XXXX
// I think "tune2fs -l name" will return both UUID and LABEL but not /dev/XXXX
// so there are 3 out of possible 4
string Dcheck::get_shortest_fs_name(string name)
{
string shortest_name;
string label_name;
string uuid_name;
string result;
string command;
guint len;

command = (string)"tune2fs -l " + name;
	
result = get_output(command);

label_name = get_label_name(result.c_str()); //These return memstrs
uuid_name = get_uuid_name(result);

shortest_name = name; // The default

	
if ( label_name.length()  && (label_name.length() <= name.length()) )
  {
  shortest_name = label_name;
  }

if ( uuid_name.length()  && (uuid_name.length() <= shortest_name.length()) )
  {
  shortest_name = uuid_name;
  }

return shortest_name;
}





void Dcheck::get_tune(string fs, string &size, string &state,
                     string &mount_point, string &last_check,
                     string &num_mounts, string &max_mounts,
                     string &interval, string &interval_type, gchar *result)
{
char *p, *t;
char c;
string data;
gboolean got_interval, got_type;

if (!result)
  {
  string command = "tune2fs -l " + fs;
  data = misc::get_output(command);
  }
else data = result;

got_interval = got_type = FALSE;
// Now get what we want from output
// size
  
size = get_fs_size(data);

// state  - Filesystem state:         clean
if ( (p = strstr((char *)data.c_str(), "Filesystem state:")) )
  {
  p += 17;
  while ((*p==' ')||(*p=='\t')) p++;
  t = p;
  while ((*t)&&((*t)!='\n')) t++;
  c = *t;
  *t = 0;
  state = p;
  *t = c;
  }
else
  {
  // ERROR
  state = "UNKNOWN";
  }

// mount_point - Last mounted on:          /boot
mount_point = get_mount_point(data);

// last_check - Last checked:             Tue May 31 16:35:03 2011
last_check = get_last_check(data);
 

  
// num_mounts - Mount count:
if ( (p = strstr((char *)data.c_str(), (char *)"Mount count:")) )
  {
  p += 12;
  while ((*p==' ')||(*p=='\t')) p++;
  t = p;
  while ((*t)&&((*t)!='\n')) t++;
  c = *t;
  *t = 0;
  num_mounts = p;
  *t = c;
  }
else
  {
  // ERROR
  num_mounts = "0";
  }


// max_mounts - Maximum mount count:
if ( (p = strstr((char *)data.c_str(), "Maximum mount count:")) )
  {
  p += 20;
  while ((*p==' ')||(*p=='\t')) p++;
  t = p;
  while ((*t)&&((*t)!='\n')) t++;
  c = *t;
  *t = 0;
  max_mounts = p;
  *t = c;
  }
else
  {
  // ERROR
  max_mounts = "-1";
  }

//===========================
// CHECK INTERVAL IS WRONG!
//===========================
	
// interval - Check interval:
if ( (p = strstr((char *)data.c_str(), "Check interval:")) )
  {

  p += 15;
  while ((*p) && (*p!=( '(' ) ) && ((*p)!='\n') ) p++;

  if ( *p==( '(' ) )                   // start bracket
    {
    p++;                               // start of bracket contents
    t = p;                             // Temp start of bracket contents

    while ((*t) && (*t!=( ')' ) )  && ((*t)!='\n') ) t++;
      {
      if ( *t==( ')' ) )
        {
        *t = 0;                        // end of bracket contents
        if (!STREQ("<none>", p))
          {
          char *type;
          char *cp = p;                // current position, first digit assumed

          while (isdigit(*cp)) cp++;
          c = *cp;
          *cp = 0;                     // end of digit string
			  
          interval = p; // Length of interval
			  
          got_interval = TRUE;
          *cp = c;
          
          while ((*cp) && ((*cp)==' ' || (*cp)=='\t')) cp++;
          type = cp;

          while (isalpha(*cp)) cp++;
          c = *cp;
          *cp = 0;
          interval_type = type;
          got_type = TRUE;
			  
          *cp = c;
			  			  
          }
        else // we got <none>
          {
          interval = "0";
          interval_type = "<none>";
          got_interval = got_type = TRUE;
  
          }
        *t = ( ')' );

        return;
        }
      }

    }
  }

// ERROR
if (!got_interval) interval = "0";
if (!got_type) interval_type = "<none>";
}




// Use GList *FS_string_list for the check
// We need to check for UUID as well as name because they can both appear in the list
// Labels as well!
// Maybe where possible change them to /dev/XXXX as it is shorter

bool Dcheck::fs_already_listed(string name, string &listed_name)
{
Lists lists;
FS_strings_info *fs_str;
list<FS_strings_info *> ::iterator i;
	
// cout << "\nDcheck::fs_already_listed - ENTERED" << endl;
	
// Check name as is
//=========================================

//while (item)

for (i=lists.FS_string_list.begin(); i!=lists.FS_string_list.end(); ++i)

	{
	fs_str = (FS_strings_info *)*i;

	if ( fs_str->name == name )
    {
		
	//	cout << "\nDcheck::fs_already_listed - GOT it in FS_string_list" << endl;
	

    listed_name = fs_str->name;
    return TRUE;
    }
  }


//cout << "\nDcheck::fs_already_listed - NOT in FS_string_list" << endl;
	


// Now if name is /dev/XXXX check if it's UUID is aleady in the list
// THEN check for LABEL its label
if (STREQN(name.c_str(), "/dev/", 4))
  {
  string command;
  string result;
  char *p, *q, *r;
  char *start;
  char c;
  string dev;

	command = "blkid " + name;
  result = get_output(command);

  // Check for UUID
  //=========================================
  p = strstr( (char *)result.c_str(), "UUID=" );
  if (p)
    {
    start = p;
    while (!isspace(*p)) p++;
    c = *p;
    *p = 0;

// This unfortunately has quotes around the actual UUID
// It has to be de-quoted somehow

    dev = start;

    q = (char *)dev.c_str();
    while (*q)
      {
      if ( (*q)=='"' )
        {
        // Move everything from here on 1 char towards the front
        r = q + 1;
        while ( (*r) )
          {
          *(r-1) = *r;
          r++;
          }
        }
      q++;
      }

		while ( dev.length() && (dev[dev.length()-1]=='"') )
			{
			dev.resize(dev.length()-1);
			}
		       
		for (i=Lists::FS_string_list.begin(); i!=Lists::FS_string_list.end(); ++i)
      {
      //fs_str = (struct FS_strings_info *)(item->data);

			fs_str = (FS_strings_info *)*i;

      if ( fs_str->name == dev )
        {
        listed_name = dev;
        return TRUE;
        }
      }

    *p = c;
    }

  // NOW check for LABEL using similar method
  //=========================================
  p = strstr( (char *)result.c_str(), "LABEL=" );
  if (p)
    {
    start = p;
    while (!isspace(*p)) p++;
    c = *p;
    *p = 0;

    dev = start;
    q = (char *)dev.c_str();
    while (*q)
      {
      if ( (*q)=='"' )
        {
        // Move everything from here on 1 char towards the front
        r = q + 1;
        while ( (*r) )
          {
          *(r-1) = *r;
          r++;
          }
        }
      q++;
      }

		while ( dev.length() && (dev[dev.length()-1]=='"') )
			{
			dev.resize(dev.length()-1);
			}

		for (i=Lists::FS_string_list.begin(); i!=Lists::FS_string_list.end(); ++i)
      {
			fs_str = (FS_strings_info *)*i;


      if ( fs_str->name == dev )
        {
        listed_name = dev;
        return TRUE;
        }
      }

    *p = c;
    }
  }

listed_name.clear();
return FALSE;
}


void Dcheck::add_to_lists(string fs_name)
{
Lists lists;
string name;

string listed_name;
GtkWidget *down_image;

// cout << "\nDcheck::add_to_lists - ENTERED,          name= " << fs_name << endl;

name = get_shortest_fs_name(fs_name); // name is now in memory

//cout << "Dcheck::add_to_lists - ENTERED, shortest name= " << name << endl;


if (!fs_already_listed(name, listed_name))
  {
  string size, state, mount_point, last_check, num_mounts, max_mounts, interval, interval_type;
  string text;
  gint type;
	string control;

//cout << "Dcheck::add_to_lists - Not already listed " << endl;


  // Now add the rest of the data with tune2fs
  get_tune(name, size, state, mount_point, last_check, num_mounts, max_mounts, interval, interval_type, NULL);

  if (interval_type == "<none>") type = 0;
  else if ((interval_type == "day") || (interval_type == "days")) type = 1;
  else if ((interval_type == "week") || (interval_type == "weeks")) type = 2;
  else if ((interval_type == "month") || (interval_type == "months")) type = 3;
  else type = 0;


	FS_widgets_info *fs = new(FS_widgets_info); // calloc(sizeof(struct FS_widgets_info), 1);

// name
  fs->name_label = gtk_label_new(name.c_str());               // Put shortest name into fs        
  gtk_misc_set_alignment(GTK_MISC(fs->name_label), 0, .5);    // align left
	text = (string)"Filesystem\n" + mount_point + "\nLast checked:\t\t" + last_check;		
  gtk_widget_set_tooltip_text(fs->name_label, text.c_str());
    
// size
  fs->size_label = gtk_label_new(size.c_str());              // Put size into fs
  gtk_misc_set_alignment(GTK_MISC(fs->size_label), 1, .5);   // align right

// state
  fs->state_label = gtk_label_new(state.c_str());             // Put state into fs
  gtk_misc_set_alignment(GTK_MISC(fs->state_label), .5, .5);  // align centre h & v
	if (state == "clean")
		{
  	// GREEN
  	decorate_button(fs->state_label, &darkgreen, NULL, NULL, NULL);
    }
  else
    {
    // RED
    decorate_button(fs->state_label, &darkred, NULL, NULL, NULL);
    }

// managed
	control = lists.get_control(name);		
	fs->managed_check_button = gtk_check_button_new();        // selected if this fs is dcheck-managed
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fs->managed_check_button), (control=="dcheck") ? TRUE: FALSE);
  gtk_widget_set_tooltip_text(fs->managed_check_button, "Tick for this filesystem to be managed by dcheck");
	g_signal_connect(G_OBJECT(fs->managed_check_button), "button_press_event", G_CALLBACK(callback_menu), this);
	g_signal_connect(G_OBJECT(fs->managed_check_button), "clicked", G_CALLBACK(callback_managed_clicked), this);
                 
// check
    fs->dcheck_button = gtk_check_button_new();
		text = (string)"Check\nTick for filesystem check at next mount\n" + mount_point + "\nLast checked:\t\t" + last_check;
    gtk_widget_set_tooltip_text(fs->dcheck_button, text.c_str());
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fs->dcheck_button), FALSE);
		g_signal_connect(G_OBJECT(fs->dcheck_button), "button_press_event", G_CALLBACK(callback_menu), this);
		g_signal_connect(G_OBJECT(fs->dcheck_button), "clicked", G_CALLBACK(callback_dcheck_clicked), this);

		
// num mounts			
    fs->num_mounts_entry = gtk_entry_new();                         // Put  num_mounts into fs
    gtk_entry_set_max_length(GTK_ENTRY(fs->num_mounts_entry), 4);
    gtk_entry_set_width_chars(GTK_ENTRY(fs->num_mounts_entry), 4);
    gtk_entry_set_text(GTK_ENTRY(fs->num_mounts_entry), num_mounts.c_str());
		fs->old_managed_num_mounts = fs->old_dcheck_num_mounts = num_mounts;
    gtk_entry_set_alignment(GTK_ENTRY(fs->num_mounts_entry), 1);     // set text to the right

		g_signal_connect(G_OBJECT(fs->num_mounts_entry), "button_press_event", G_CALLBACK(callback_menu), this);
		g_signal_connect(G_OBJECT(fs->num_mounts_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)0);
    gtk_widget_set_tooltip_text(fs->num_mounts_entry,
                                  "Mounts\n"
                                  "The number of times this filesystem has been mounted without a check");


// It appears that g_signal_connect has to come last...

// max mounts			
    fs->max_mounts_entry = gtk_entry_new();                          // Put max_mounts into fs
    gtk_entry_set_max_length(GTK_ENTRY(fs->max_mounts_entry), 4);
    gtk_entry_set_width_chars(GTK_ENTRY(fs->max_mounts_entry), 4);
    gtk_entry_set_text(GTK_ENTRY(fs->max_mounts_entry), max_mounts.c_str());
		fs->old_managed_max_mounts = fs->old_dcheck_max_mounts = max_mounts;
    gtk_entry_set_alignment(GTK_ENTRY(fs->max_mounts_entry), 1);      // set text to the right
 		g_signal_connect(G_OBJECT(fs->max_mounts_entry), "button_press_event", G_CALLBACK(callback_menu), this);
 		g_signal_connect(G_OBJECT(fs->max_mounts_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)1);
    gtk_widget_set_tooltip_text(fs->max_mounts_entry,
                                  "MaxMounts\n"
                                  "The maximum number of times this filesystem can been mounted without a check.\n"
                                  "Set to -1 or 0 to disable.\n"
                              		"NOTE: -1 or 0 also seems to disabled interval checks.");


 // interval       
		fs->interval_entry = gtk_entry_new();                      // Put interval into fs
    gtk_entry_set_max_length(GTK_ENTRY(fs->interval_entry), 4);
    gtk_entry_set_width_chars(GTK_ENTRY(fs->interval_entry), 4);
    gtk_entry_set_text(GTK_ENTRY(fs->interval_entry), interval.c_str());
    gtk_entry_set_alignment(GTK_ENTRY(fs->interval_entry), 1); // set text to the right
 		g_signal_connect(G_OBJECT(fs->interval_entry), "button_press_event", G_CALLBACK(callback_menu), this);
    g_signal_connect(G_OBJECT(fs->interval_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)0);
		gtk_widget_set_tooltip_text(fs->interval_entry,
                                  "Interval\n"
                                  "The maximum time between checks.\n"
                                  "Set to 0 to disable.");



 // interval type     

		fs->interval_type_button = gtk_button_new_with_label(interval_type_txt[type]);

		g_signal_connect(fs->interval_type_button, "button_press_event", G_CALLBACK(callback_menu), this);  

		gtk_widget_set_tooltip_text(fs->interval_type_button,
                                  "Interval Type\n"
                                  "The interval may be day(s) week(s) or month(s).\n"
                                  "Set to <none> to disable.");

// down image
		fs->select_button = gtk_button_new();
		
		down_image = gtk_image_new_from_icon_set(gtk_icon_set_new_from_pixbuf(down_pixbuf), 
                                                GTK_ICON_SIZE_BUTTON);
	
// if (!down_image) cout << "Dcheck::add_to_lists - NO down_image" << endl;
		
		gtk_button_set_image(GTK_BUTTON(fs->select_button), down_image);
		
		gtk_button_set_always_show_image(GTK_BUTTON(fs->select_button), TRUE); 
		gtk_widget_set_tooltip_text(fs->select_button,
                                  "Interval Type\n"
                                  "The interval may be day(s) week(s) or month(s).\n"
                                  "Set to <none> to disable.");
		g_signal_connect(G_OBJECT(fs->select_button), "button_press_event", G_CALLBACK(callback_menu), this);
 

		
// Add fs to the list
	lists.FS_widget_list.push_back(fs);       // everything now in list


		
// This is a list of their original values, needed if fixing
//cout << "Dcheck::add_to_lists - ADDING " << name << endl;
		
  FS_strings_info *fs_str = new(FS_strings_info); // (struct FS_strings_info *)calloc(sizeof(struct FS_strings_info), 1);

//if (fs_str) cout << "Dcheck::add_to_lists - GOT fs_str" << endl;
//else cout << "Dcheck::add_to_lists - NOT GOT fs_str" << endl;

		
	fs_str->name = name;
  fs_str->size = size;
  fs_str->state = state;
  fs_str->control = ((control.length()) ? control :  "tune2fs");  // "tune2fs" is default
	fs_str->check_wanted = FALSE;       // at startup
  fs_str->mount_point = mount_point;
  fs_str->last_check = last_check;
  fs_str->num_mounts = num_mounts;
  fs_str->max_mounts = max_mounts;
  fs_str->interval = interval;
  fs_str->interval_type = interval_type;

			
	Lists::FS_string_list.push_back(fs_str);
//	Lists::FS_start_string_list.push_back(fs_str);
		
  }
else
  {

  // Already listed. Is this name shorter than listed name?
  if (listed_name.length() && (fs_name.length() < listed_name.length()) )
    {
		// Change fs_str->name and fs->name_label
    FS_strings_info *fs_str = lists.get_fs_str(listed_name);
    fs_str->name = fs_name;
    }
  }
}








/*
struct mntent {
               char *mnt_fsname;   // name of mounted file system
               char *mnt_dir;      // file system path prefix
               char *mnt_type;     // mount type (see mntent.h)
               char *mnt_opts;     // mount options (see mntent.h)
               int   mnt_freq;     // dump frequency in days
               int   mnt_passno;   // pass number on parallel fsck
           };
*/

// Use getmntent() to read fstab for filesystems.
// Then make sure each is a Linux filesystem with tune2fs -l device
// and check the output for a problem
// This doesn't get other filesystems such as /media/XXXXX
// Perhaps a combination of this and "fdisk -l | grep /dev"
void Dcheck::get_filesystems_fstab(void)
{
struct mntent *mnt;

FILE *fp = setmntent("/etc/fstab", "rb");

while ( (mnt=getmntent(fp)) )
  {
  string result;
  string command;
	string fsname = mnt->mnt_fsname;

	// skip these ones
	if ( (fsname=="tmpfs") || (fsname=="devpts") ||
	    (fsname=="sysfs")  || (fsname=="proc")   || 
	    ((string)mnt->mnt_type==MNTTYPE_SWAP) )
		{
		continue;
		}

		
	// Check if this filesystem is Linux
  command = (string)"tune2fs -l " + fsname;

  result = get_output(command);

		
	if ( (get_os_type(result)) == "Linux" )
		{
    // This is a Linux filesystem so get everything....
    add_to_lists(mnt->mnt_fsname); 
		}

  }
endmntent(fp);
}


void Dcheck::get_filesystems(void)
{
get_filesystems_fstab();


string output = get_output("fdisk -l | grep /dev");
		


char *p = (char *)output.c_str();   // current position in output

//cout << "Dcheck::get_filesystems - DONE get_output(\"fdisk -l | grep /dev\")" << endl;

		

		
// Now extract the relevant Linux filesystems
while(*p)
  {    
  p = strstr(p, "/dev");

  if ( p && (*(p-1) == '\n') )  // if /dev is found at start of line
    {
    // This line is a filesystem_list
    // Is the last word of this line "Linux"?
    char *s;
    char temp;
    char *line = p;
      

    while ( *p && ((*p) != '\n')) p++;   // find eol
    temp = *p;                           // save the character
    *p = 0;                              // eol

    s = strstr(line, "Linux");
    if ( s && (*(s+5)==0) )   // strlen("Linux") is 5, this is the end of the line
      {
      // FOUND A LINUX FILESYSTEM, so extract its partition
      char *c = line;
      char t;

      while ( ( (*c)!=' ') && ((*c)!='\t') ) c++;  // Find the space after the fs name
      t = *c;                  // save the character
      *c = 0;

      // ==============================================================================
      // "line" is now this fs name...
      // If "line" is not already in fs then add it otherwise skip
      // MAKE THIS A FUNCTION called from here and from get_filesystems_fstab
      // In fact the new function could check that the fs hadn't already been added...
      // ==============================================================================

//cout << "Dcheck::get_filesystems - ADDING line= " << line << endl;

      add_to_lists(line);


//cout << "Dcheck::get_filesystems -   DONE line= " << line << endl;

      *c = t;                                   // restore the character


      *p = temp;                    // restore the character
      while (*p == '\n') p++;       // start new line   
      }

    else // it wasn't Linux'
      {
      *p = temp;                           // restore the character
      while ((*p) && ((*p)!='\n')) p++;    // go to end of this line
      while ((*p)=='\n') p++;              // start new line      
      }
    
    }
  else // found /dev but not at line start
    {
    while ((*p) && ((*p)!='\n')) p++;    // go to end of this line
    while ((*p)=='\n') p++;              // start new line
    }

  } // end of while


  
}







/*******************************************************
 * Handler for left and right mouse button popup menus *
 *******************************************************/

gint Dcheck::callback_menu(GtkWidget *widget, GdkEventButton *event, Dcheck*dc)
{
Lists lists;
	
//cout << "Dcheck::callback_menu - ENTERED" << endl;
// RIGHT MOUSE BUTTON
if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) )
  {

	//cout << "Dcheck::callback_menu - got right mouse button" << endl;

  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, dc,
                        event->button, event->time);
  return TRUE;
  }

// LEFT MOUSE BUTTON
if( lists.is_fs_w_type(widget) && (event->type == GDK_BUTTON_PRESS) && (event->button == 1) )
  {

	dc->interval_type_widget = widget;

	gtk_menu_popup (GTK_MENU (menu_left), NULL, NULL, NULL, dc,
                        event->button, event->time);
  return TRUE;
  }
	

return FALSE;
}



void Dcheck::create_menu(void)
{
GtkWidget *menu_item;

//
// Array of pointers to functions - can't use this in C++
//static void (*action[6])(GtkWidget *widget, gpointer data) =
//  {
//  callback_ok,
//  callback_change_last_check,
//  callback_change_auth,
//  callback_change_status_window,
//  callback_about,
//  callback_quit
//  };


//cout << "Dcheck::create_menu - ENTERED" << endl;


//cout << "Dcheck::create_menu - config.su_auth= " << config.su_auth << endl;

action_txt[2] = (config.su_auth) ? (char *)"Change Authorisation to \"sudo\"" : (char *)"Change Authorisation to \"su\"";
action_txt[3] = (config.status_wanted) ? (char *)"Disable status window" : (char *)"Enable status window";


//cout << "\nDcheck::create_menu - " << action_txt[3] << endl;
	
menu = gtk_menu_new ();

for (int i = 0; i < 6;  i++)
  {

            // Create a new menu-item with action name...
menu_item = gtk_menu_item_new_with_label (action_txt[i]);

            // ...and add it to the menu.
//  gtk_menu_append (GTK_MENU (menu), menu_item);
		
// USE:	gtk_menu_shell_append(GtkMenuShell *menu_shell,GtkWidget *child);
		
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);



//cout << "Dcheck::create_menu - ADDED " << action_txt[i] << endl;

	
		
// do the action associated with this action name 


	switch (i)
		{
		case 0:
		g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_apply), this);
		break;
				
		case 1:
		g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_change_last_check), this);
		break;

		case 2:
		g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_change_auth), this);
		break;

		case 3:
		g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_change_status_window), this);
		break;

		case 4:
		g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_about), this);
		break;

		case 5:
		g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_quit), this);
		break;
		}


// Show the widget
  gtk_widget_show (menu_item);
	
  }

}



/*************************************
 * Create menu for left mouse button *
 * on "Interval Type" ONLY           *
 *************************************/

void Dcheck::create_menu_left(void)
{
GtkWidget *menu_item;

menu_left = gtk_menu_new ();

for (int i = 0; i < 4;  i++)
  {

// Create a new menu-item with action name...
	menu_item = gtk_menu_item_new_with_label(interval_type_txt[i]);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_left), menu_item);


// Show the widget
  gtk_widget_show (menu_item);

switch(i)
		{
	case 0:
	g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_none), this);
	break;

	case 1:
	g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_days), this);
	break;

	case 2:
	g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_weeks), this);
	break;


	case 3:
	g_signal_connect_swapped(G_OBJECT (menu_item), "activate", GCallback(callback_months), this);
	break;

		}

	}
}















void Dcheck::dcheck_gui(int argc, char *argv[])
{	
GdkPixbuf *dtime_pixbuf_scaled;
GtkRequisition req_minimum, req_natural;
GtkWidget *vbox, *hbox, *box;
GtkWidget *scrolled_window;
GtkWidget *wid;
GtkWidget *table;
Lists lists;
list<FS_widgets_info *>::iterator i;
string fs_name, control;
FS_widgets_info *fs = NULL;
char *pathname = argv[0];


//cout << "Dcheck::dcheck_gui - Entered" << endl;


	
gtk_init(&argc, &argv);

config.set_status(&status);
config.load_conf();
status.status_wanted = config.status_wanted;

down_pixbuf = gdk_pixbuf_new_from_inline(-1, down_pixbuf_src, FALSE, NULL);

// This has to come AFTER config.load_conf()
get_filesystems();



for (int i=0; i<argc; i++)
	{
//	cout << "Dcheck::dcheck_gui - argv[" << i << "]= " << argv[i] << endl;

	if ((string)argv[i] == "-f")
		{
			
		config.fix_dcheck();

		}
	}

window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_window_set_title (GTK_WINDOW (window), "DCheck");

g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (callback_destroy),
		    this);
  
    
gtk_window_set_default_size(GTK_WINDOW(window), 650, 400); // width, height
  
dcheck_icon_pixbuf = gdk_pixbuf_new_from_inline(-1,
                                            dcheck_icon_pixbuf_src,
					                                  FALSE, NULL);

  

dcheck_icon_image = gtk_image_new_from_icon_set(gtk_icon_set_new_from_pixbuf(dcheck_icon_pixbuf), 
                                                GTK_ICON_SIZE_BUTTON);
		
gtk_window_set_icon( GTK_WINDOW(window), dcheck_icon_pixbuf);

gtk_widget_add_events(GTK_WIDGET(window), GDK_BUTTON_PRESS_MASK);  // Needed for the popup menu
g_signal_connect(window, "button_press_event", G_CALLBACK(callback_menu), this);  
create_menu();
create_menu_left();



// We need a scrollable window
vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
	
gtk_container_add(GTK_CONTAINER(window), vbox);


scrolled_window = gtk_scrolled_window_new(NULL, NULL);
gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolled_window),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
					box);


	
	
hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
                        
GtkWidget *column[8];
for (int n=0; n<8; ++n)
	{
	column[n] = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
	gtk_box_set_homogeneous(GTK_BOX(column[n]), TRUE); //(n<7) ? TRUE: FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), column[n], TRUE, TRUE, 0); // expand, fill, padding
	}

gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0); // expand, fill, padding






	
//----------
// HEADINGS
//----------
wid = gtk_label_new("Filesystem");
gtk_misc_set_alignment(GTK_MISC(wid), 0, 0.5);
gtk_box_pack_start(GTK_BOX(column[0]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Size");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[1]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("State");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[2]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Managed");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[3]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Check");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[4]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Mounts");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[5]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("MaxMount");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[6]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Interval");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[7]), wid, FALSE, FALSE, 0); // expand, fill, padding

	
	
for (i=Lists::FS_widget_list.begin(); i!=Lists::FS_widget_list.end(); ++i)
	{
			
	FS_widgets_info *fs = (FS_widgets_info *)*i;


		
  if (!fs)
    {
    continue;
    }

	int col = 0;
		
//------------
// FILESYSTEM
//------------
	gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(fs->name_label), FALSE, FALSE, 0); // expand, fill, padding
		
//------------
// SIZE
//------------
		gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(fs->size_label), FALSE, FALSE, 0); // expand, fill, padding
 
//------------
// STATE
//------------
	gtk_box_pack_start(GTK_BOX(column[col++]), fs->state_label, FALSE, FALSE, 0); // expand, fill, padding




//------------
// Managed
//------------
	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
	gtk_box_set_homogeneous(GTK_BOX(box), TRUE);
	gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(fs->managed_check_button), FALSE, FALSE, 0); // expand, fill, padding
	gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(box), TRUE, TRUE, 0); // expand, fill, padding


		
//------------
// Check
//------------
	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
	gtk_box_set_homogeneous(GTK_BOX(box), TRUE);
	gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(fs->dcheck_button), FALSE, FALSE, 0); // expand, fill, padding
	gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(box), FALSE, FALSE, 0); // expand, fill, padding

//------------
// # MOUNTS
//------------
	gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(fs->num_mounts_entry), FALSE, FALSE, 0); // expand, fill, padding


//------------
// MAXMOUNTS
//------------
	gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(fs->max_mounts_entry), FALSE, FALSE, 0); // expand, fill, padding


    
//------------
// INTERVAL
//------------
	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
  gtk_box_pack_start(GTK_BOX(box), fs->interval_entry, FALSE, TRUE,0);     // interval
		
	wid = gtk_alignment_new (1,1,1,1);
	gtk_box_pack_start(GTK_BOX(box), wid, TRUE, TRUE,0); // type

	gtk_widget_get_preferred_size(GTK_WIDGET(wid), &req_minimum, &req_natural);
	gtk_widget_set_size_request(GTK_WIDGET(fs->interval_type_button), req_natural.width, req_natural.height);
	gtk_container_add (GTK_CONTAINER(wid), fs->interval_type_button);

	gtk_box_pack_start(GTK_BOX(box), fs->select_button, FALSE, TRUE,0);

	gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(box), FALSE, FALSE, 0); // expand, fill, padding


	                   
	}  // -------------- END of while (item) -----------------------

		
box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
gtk_button_box_set_layout(GTK_BUTTON_BOX(box), GTK_BUTTONBOX_EDGE);// GTK_BUTTONBOX_SPREAD);
gtk_box_set_spacing(GTK_BOX(box), 0);
gtk_box_pack_end(GTK_BOX(vbox), GTK_WIDGET(box), FALSE, FALSE, 0); // expand, fill, padding


apply_button = gtk_button_new_with_mnemonic("_Apply");
	
//gtk_button_set_image(GTK_BUTTON(apply_button), ok_image);

gtk_button_set_image(GTK_BUTTON(apply_button),gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON));
	
decorate_button(GTK_WIDGET(apply_button), &darkgreen, &lightyellow, NULL, NULL);        

// Make sure the image is displayed!
gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(apply_button)))); 

g_signal_connect(G_OBJECT(apply_button), "button_press_event",
                 G_CALLBACK(callback_menu), this);  
g_signal_connect(G_OBJECT(apply_button), "clicked",
		    	G_CALLBACK(callback_apply),  this);

gtk_widget_set_tooltip_text (apply_button,
      "Apply\n"
      "Set the chosen values.\nSave configuration to \"/etc/dcheck.conf\".");
gtk_container_add(GTK_CONTAINER(box), apply_button);




// NOTE: req_minimum or req_natural can be NULL
gtk_widget_get_preferred_size(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(apply_button))), //GTK_WIDGET(ok_image),
                              &req_minimum, &req_natural);

dtime_pixbuf = gdk_pixbuf_new_from_inline(-1,
                                          dtime_icon_pixbuf_src,
					                                FALSE, NULL);
	
dtime_pixbuf_scaled = gdk_pixbuf_scale_simple(dtime_pixbuf, req_natural.width, req_natural.height, GDK_INTERP_BILINEAR);
dtime_image = gtk_image_new_from_pixbuf(dtime_pixbuf_scaled);




	
change_last_check_button = gtk_button_new_with_mnemonic("_Change Last Check");
decorate_button(GTK_WIDGET(change_last_check_button), &darkblue,  &lightyellow, NULL, NULL);
  
gtk_button_set_image(GTK_BUTTON(change_last_check_button), dtime_image);

gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(change_last_check_button))));

g_signal_connect(G_OBJECT(change_last_check_button), "button_press_event",
               		G_CALLBACK(callback_menu), this);   
g_signal_connect(G_OBJECT(change_last_check_button), "clicked",
		    					G_CALLBACK(callback_change_last_check),  this);
 if(!gtk_widget_get_has_tooltip(change_last_check_button))
  {
  gtk_widget_set_has_tooltip(change_last_check_button, TRUE);
  }
gtk_widget_set_tooltip_text (change_last_check_button,
      "Change Last Check\n"
      "Alter when filesystem(s) were last checked");
gtk_container_add(GTK_CONTAINER(box), change_last_check_button);


quit_button = gtk_button_new_with_mnemonic("_QUIT");
gtk_button_set_image(GTK_BUTTON(quit_button), gtk_image_new_from_stock(GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON));
  
decorate_button(GTK_WIDGET(quit_button), &darkred,  &lightyellow, NULL, NULL);

// Make sure the image is displayed!
gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(quit_button)))); 


g_signal_connect(G_OBJECT(quit_button), "button_press_event", G_CALLBACK(callback_menu), this);  
g_signal_connect(G_OBJECT(quit_button), "clicked", G_CALLBACK(callback_quit), this);

gtk_widget_set_tooltip_text(quit_button,
      "Quit\n"
      "Terminate the program");
  
gtk_container_add(GTK_CONTAINER(box), quit_button);


  
gtk_widget_show_all(window);
  
gtk_main ();

cout <<"Goodbye world" << endl;

exit(EXIT_SUCCESS);
}



