/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       dcheck.h                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       02-Mar-2013 11.54.02 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Header file for dcheck.cpp in dcheck_main                                 *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __DCHECK_H__
#define __DCHECK_H__

#include <gtk/gtk.h>

#include "about.h"
#include "config.h"
#include "data.h"
#include "last.h"
#include "lists.h"

#include "status.h"
#include "misc.h"


//using namespace misc;

class Dcheck
{
public:
	Dcheck(void);
	~Dcheck(void);
	
	void dcheck_gui(int argc, char *argv[]);
	

	static class About      *about;
	static class Last_check *last_check;

	static GtkWidget *interval_type_widget; // for callbacks of left popup menu

private:
	static void callback_destroy(GtkWidget *widget, Dcheck *dc);
	static void callback_quit(GtkWidget *widget, Dcheck *dc);
	static void callback_apply(GtkWidget *widget, Dcheck *dc);
	static void callback_change_auth(GtkWidget *widget, Dcheck *dc);
	static void callback_change_last_check(GtkWidget *widget, Dcheck *dc);
	static void callback_about(GtkWidget *widget, Dcheck *dc);

	static void callback_change_status_window(GtkWidget *widget, Dcheck *dc);
	static void callback_managed_clicked(GtkWidget *widget, Dcheck *dc);
	static void callback_dcheck_clicked(GtkWidget *widget, Dcheck *dc);

	// Left Menu
	void interval_type_changed(FS_widgets_info *fs_w, string old_type, int new_type, Dcheck*dc);

	static void callback_none(GtkWidget *widget, Dcheck*dc);
	static void callback_days(GtkWidget *widget, Dcheck*dc);
	static void callback_weeks(GtkWidget *widget, Dcheck*dc);
	static void callback_months(GtkWidget *widget, Dcheck*dc);







	static gint callback_menu(GtkWidget *widget, GdkEventButton *event, Dcheck *dc);
	static void callback_insert_text_handler(GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data);

	class FS_widgets_info *get_fs_of_widget(GtkWidget *widget);

	void create_menu(void);
	void create_menu_left(void);
	
	bool fs_already_listed(string name, string &listed_name);
	void add_to_lists(string fs_name);
	string get_fs_size(string data);
	string get_label_name(string data);
	string get_uuid_name(string data);
	string get_shortest_fs_name(string name);
	void get_filesystems_fstab(void);
	void get_filesystems(void);
	void get_tune(string fs, string &size, string &state,
                     string &mount_point, string &last_check,
                     string &num_mounts, string &max_mounts,
                     string &interval, string &interval_type, gchar *result);

	static GtkWidget *window;
	static GtkWidget *menu;
	static GtkWidget *menu_left;

	static GtkWidget *apply_button;
	static GtkWidget *change_last_check_button;
	static GtkWidget *quit_button;


	static GdkPixbuf *dtime_pixbuf;
	static GtkWidget *dtime_image;

	static GdkPixbuf *dcheck_icon_pixbuf;
	static GtkWidget *dcheck_icon_image;

	static GdkPixbuf *down_pixbuf;
	
	static char *action_txt[6];
	static char *interval_type_txt[4];

	
	Config config;
	Status status;


protected:
	

};

#endif // __DCHECK_H__
