/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       last.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       05-Mar-2013 18.53.19 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Window for changing the date/time of the last check of a filesystem       *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#define _XOPEN_SOURCE /* glibc2 needs this */
#include <time.h>
#include <string.h>
#include <stdio.h>

#include "status.h"
#include "last.h"

list<FS_last_check*> Last_check::FS_last_check_list;

GdkPixbuf *Last_check::dtime_pixbuf      = NULL;

Last_check::Last_check(void)
{
//cout << "Last_check::last_check" << endl;
}

Last_check::~Last_check(void)
{
//cout << "Last_check::~Last_check" << endl;


//if (FS_last_check_list) g_list_free(FS_last_check_list);
//FS_last_check_list = NULL;
FS_last_check_list.clear();

if (dtime_pixbuf)
	{
	g_object_unref(G_OBJECT(dtime_pixbuf));
	dtime_pixbuf = NULL;
	}
dcheck->last_check = NULL;
gtk_main_quit();
while (gtk_events_pending ())
  gtk_main_iteration ();

//cout << "Last_check::~Last_check - LEAVING" << endl; 

}


void Last_check::callback_last_check_destroy(GtkWidget *widget, Last_check *lc)
{
//cout << "Last_check::callback_last_check_destroy - ENTERED" << endl; 
delete lc;
}

void Last_check::callback_last_check_cancel(GtkWidget *widget, Last_check *lc)
{
gtk_widget_destroy(GTK_WIDGET(lc->last_check_window));

}

// return index into FS_last_check_list of this widget or -1 if not in list
FS_last_check *Last_check::get_widget_FS_last_check(GtkWidget *widget) 
{
FS_last_check *fs_lc;
list<FS_last_check *>::iterator i;
	

for (i=FS_last_check_list.begin(); i!=FS_last_check_list.end(); ++i)

	{
  fs_lc = (FS_last_check *)*i;
		
  if (  (fs_lc->last_check_label == widget) || 
        (fs_lc->day_entry == widget)        ||
        (fs_lc->month_entry == widget)      ||
        (fs_lc->year_entry == widget)       ||
        (fs_lc->hour_entry == widget)       ||
        (fs_lc->minute_entry == widget)     ||
        (fs_lc->second_entry == widget)     ||
        (fs_lc->now_check_button == widget)  )
    {
    return fs_lc;
    }
  }
return NULL;
}

void Last_check::callback_last_check_ok(GtkWidget *widget, Last_check *lc)
{
Lists lists;
char *last_check_txt;
char *t;	
string command;
struct tm tm;
struct tm * tm_now;
time_t now;
int year;
int month;
int day;
int hour;
int minute;
int second;

string result;
string last_check;
string text;
string mount_point;
list<FS_last_check *>::iterator i;

FS_last_check *fs_lc;


// Iterate through the list and see if anything has changed
// If so, tune2fs the new value and put new value in the label
// status.fix_status( (gchar *)"Change Last Check Window", TRUE);


if  ( ( (now	= time(NULL)) != -1 )   && 
      ( (tm_now = localtime(&now)) )  && 
   		( ( t = asctime(tm_now) ) ) )
	{
	lc->status->fix_status((string)t, TRUE);
	}

for (i=FS_last_check_list.begin(); i!=FS_last_check_list.end(); ++i)
	{
	tm_now = NULL;
		
	fs_lc = (FS_last_check *)*i;
		
	lc->status->fix_status((string)"Dcheck: Last Check Window - " + fs_lc->fs_name, TRUE);
		
// Convert label to YYYYMMDDHHMMSS
    last_check_txt = (char *)gtk_label_get_text(GTK_LABEL(fs_lc->last_check_label) );


    
// Convert last_check_txt to YYYYMMDD[HH[MM[SS]]] format
// Can we use strptime()?
    memset(&tm, 0, sizeof(struct tm));
      
		// Get broken down time into tm
		if (strptime(last_check_txt, "%A %B %d %T %Y", &tm))
                  // weekdayname monthname, decimaldayinmonth, time, decimalyear
      {


// Now get the new value

// If "now" check button, use the current time
			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs_lc->now_check_button))) // if now
				{

				if  ( ( (now	= time(NULL)) != -1 )   && 
    					( (tm_now = localtime(&now)) ) )

					{
					year   = tm_now->tm_year +1900;
					month  = tm_now->tm_mon + 1;
					day    = tm_now->tm_mday;
					hour   = tm_now->tm_hour;
					minute = tm_now->tm_min;
					second = tm_now->tm_sec;
					}
				else
					{
					lc->status->fix_status("NO CHANGE", FALSE);
					lc->status->fix_status("(Failed to obtain current time!", FALSE);
					continue;
					}
				}

			else
				{				
    		year = atoi((char *)gtk_entry_get_text(GTK_ENTRY(fs_lc->year_entry)));
    		month = atoi((char *)gtk_entry_get_text(GTK_ENTRY(fs_lc->month_entry)));
    		day = atoi((char *)gtk_entry_get_text(GTK_ENTRY(fs_lc->day_entry)));
    		hour = atoi((char *)gtk_entry_get_text(GTK_ENTRY(fs_lc->hour_entry)));
    		minute = atoi((char *)gtk_entry_get_text(GTK_ENTRY(fs_lc->minute_entry)));
    		second = atoi((char *)gtk_entry_get_text(GTK_ENTRY(fs_lc->second_entry)));
				}
	
      if ( (year !=tm.tm_year+1900) ||
         (month != tm.tm_mon+1)   ||
         (day != tm.tm_mday)      ||
         (hour != tm.tm_hour)     ||
         (minute != tm.tm_min)    ||
         (second != tm.tm_sec)     )
        {
        // something has changed so fix it
        // The values have to be changed in this label and in strings

        // Get the filesystem from strings list        

// How can it be formatted?
				command = (string)"tune2fs -T " +
															convert_i2str(year,   4) +
															convert_i2str(month,  2) +
															convert_i2str(day,    2) +
															convert_i2str(hour,   2) +
															convert_i2str(minute, 2) +
															convert_i2str(second, 2) +
						                  " "                      +
															fs_lc->fs_name.c_str();
					
				result = get_output(command);


        if ( strstr((char *)result.c_str(), "Setting time filesystem last checked to ") )
          {
          // I think we should now list it so we get what actually happened
						
					command = (string)"tune2fs -l  " + fs_lc->fs_name;
          result = get_output(command);

           
          last_check = get_last_check(result);  // Does this come back OK?


//cout << "Last_check::callback_last_check_ok - last_check= " << last_check << endl;

					if (last_check != gtk_label_get_text( GTK_LABEL(fs_lc->last_check_label) ) )
						{
//						cout << "Last_check::callback_last_check_ok - HAS CHANGED" << endl;

						lc->status->fix_status((string)"\tLAST CHECK CHANGED", TRUE);
						lc->status->fix_status((string)"\tFROM:\t" + gtk_label_get_text( GTK_LABEL(fs_lc->last_check_label)),TRUE);
						lc->status->fix_status((string)"\tTO:  \t" + last_check,TRUE);


						mount_point = get_mount_point(result);

        

        		// Check tooltip
        		text = "Check\nTick for filesystem check at next mount\n" + mount_point + "\nLast checked:\t\t" + last_check;
          
						FS_widgets_info *fs_w = lists.get_fs_w(fs_lc->fs_name);
						gtk_widget_set_tooltip_text(GTK_WIDGET(fs_w->dcheck_button), text.c_str());
						gtk_widget_set_tooltip_text(GTK_WIDGET(fs_w->name_label), text.c_str());
					

						// And the last check itself in FS_string_list

						FS_strings_info * fs_str = lists.get_fs_str(fs_lc->fs_name );
						if (fs_str) fs_str->last_check = last_check;

        		// Label
        		gtk_label_set_text(GTK_LABEL(fs_lc->last_check_label), last_check.c_str()); // Is this right?


						// If "now" then Day Month Year H M S all need to be changed as well

						if (tm_now)
							{
							GtkEntryBuffer *entry_buffer;


// Use an entry buffer instead of gtk_entry_set_text

							// DAY				
							entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_lc->day_entry));
							gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
							gtk_entry_buffer_insert_text(entry_buffer, 0, convert_i2str(tm_now->tm_mday, 2).c_str(), -1); // insert text

							// MONTH				
							entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_lc->month_entry));
							gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
							gtk_entry_buffer_insert_text(entry_buffer, 0, convert_i2str(tm_now->tm_mon+1, 2).c_str(), -1); // insert text



							// YEAR				
							entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_lc->year_entry));
							gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
							gtk_entry_buffer_insert_text(entry_buffer, 0, convert_i2str(tm_now->tm_year+1900, 4).c_str(), -1); // insert text




							// HOUR				
							entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_lc->hour_entry));
							gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
							gtk_entry_buffer_insert_text(entry_buffer, 0, convert_i2str(tm_now->tm_hour, 2).c_str(), -1); // insert text



  						// MINUTE			
							entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_lc->minute_entry));
							gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
							gtk_entry_buffer_insert_text(entry_buffer, 0, convert_i2str(tm_now->tm_min, 2).c_str(), -1); // insert text




   						// SECOND			
							entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(fs_lc->second_entry));
							gtk_entry_buffer_delete_text(entry_buffer, 0, -1);                 // delete all the characters in the buffer
							gtk_entry_buffer_insert_text(entry_buffer, 0, convert_i2str(tm_now->tm_sec, 2).c_str(), -1); // insert text


							}


						}
					else
						{
						lc->status->fix_status((string)"command was:      \"" + command + "\"", FALSE);
						lc->status->fix_status((string)"command returned: \"" + result + "\"", FALSE);
						lc->status->fix_status("NO CHANGE", FALSE); 
						}

          }

        }
      else
        {
        // Status of last check for this fs is unchanged
				lc->status->fix_status("NO CHANGE", FALSE);                
        }
      }
    else
      {
      // ERROR with strptime - failed to match strptime(last_check_txt, "%A %B %d %T %Y", &tm))
                                                       // weekdayname monthname, decimaldayinmonth, time, decimalyear"

      result = fs_lc->fs_name + "\n   ---   FAILED to get a match with strptime";

			lc->status->fix_status(result, TRUE);
      }
  }

lc->status->add_status_underline();
}

void Last_check::callback_now_check_button_toggled(GtkWidget *widget, Last_check *lc)
{

FS_last_check *fs_lc = lc->get_widget_FS_last_check(widget);

  


if (fs_lc) // if found
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
    // ghost the entry widget
    gtk_widget_set_sensitive(fs_lc->day_entry, FALSE);
    gtk_widget_set_sensitive(fs_lc->month_entry, FALSE);
    gtk_widget_set_sensitive(fs_lc->year_entry, FALSE);
    gtk_widget_set_sensitive(fs_lc->hour_entry, FALSE);
    gtk_widget_set_sensitive(fs_lc->minute_entry, FALSE);
    gtk_widget_set_sensitive(fs_lc->second_entry, FALSE);
    } 
  else
    {
    // unghost the entry widget
    gtk_widget_set_sensitive(fs_lc->day_entry, TRUE);
    gtk_widget_set_sensitive(fs_lc->month_entry, TRUE);
    gtk_widget_set_sensitive(fs_lc->year_entry, TRUE);
    gtk_widget_set_sensitive(fs_lc->hour_entry, TRUE);
    gtk_widget_set_sensitive(fs_lc->minute_entry, TRUE);
    gtk_widget_set_sensitive(fs_lc->second_entry, TRUE);

    }
  }
}




static void callback_insert_text_handler(
                          GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
GtkEditable *editable = GTK_EDITABLE(entry);
int i=0, count=0;
GtkEntryBuffer *entry_buffer;
string start_text;             // a copy of the text already in the buffer
string end_text;               // a copy of the text in the altered buffer
int val;
bool error_flag = FALSE;

  
string result;

entry_buffer = gtk_entry_get_buffer(entry);
start_text = gtk_entry_buffer_get_text(entry_buffer);

// Only decimal digits are accepted
for(; i < length; i++)
	{
	if (!isdigit(text[i])) continue;
	result[count++] = text[i++];
	}

// result is the text to insert

	
if (count > 0)
  {
  g_signal_handlers_block_by_func (G_OBJECT (editable),
                                     (gpointer)callback_insert_text_handler,
                                     data);

  gtk_editable_insert_text (editable, result.c_str(), count, position);
		
  g_signal_handlers_unblock_by_func (G_OBJECT (editable),
                                       (gpointer)callback_insert_text_handler,
                                       data);
  }


g_signal_stop_emission_by_name(G_OBJECT(editable), "insert_text");

end_text = gtk_entry_buffer_get_text(entry_buffer);

// CHECK that the new version is legal...
// Convert to int
val = atoi(end_text.c_str());

if (!strcmp( (char *)data, "D" )) // Day
  {
  if (val<1 || val>31) error_flag = TRUE;
  }
else if (!strcmp( (char *)data, "M" )) // Month
  {
  if (val<1 || val>12) error_flag = TRUE;
  }
else if (!strcmp( (char *)data, "Y" )) // Year
  {
  if (val<0 || val>9999) error_flag = TRUE;
  }
else if (!strcmp( (char *)data, "m" )) // Minute
  {
  if (val<0 || val>59) error_flag = TRUE;
  }
else if (!strcmp( (char *)data, "s" )) // Second
  {
  if (val<0 || val>59) error_flag = TRUE;
  }

if (error_flag)
  {
  // Restore the original values
  gtk_entry_buffer_delete_text(entry_buffer, 0, -1); // delete all the characters in the buffer
  gtk_entry_buffer_insert_text(entry_buffer, 0, start_text.c_str(), -1); //re-insert original text
  }

}





void Last_check::change_last_check_gui(GtkWidget *widget,  Dcheck *dc, Status *st)
{
Lists lists;
GtkWidget *table;
GtkWidget *wid;
GtkWidget *scrolled_window;
GtkWidget *vbox, *box, *hbox;
FS_strings_info * fs_str;
list<FS_strings_info *>::iterator i;
int col;

struct tm tm;
	
 
//cout << "Last_check::change_last_check_gui - ENTERED" << endl;
	


dcheck = dc;
status = st;
last_check_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
g_signal_connect (G_OBJECT (last_check_window), "destroy",
		    G_CALLBACK (callback_last_check_destroy),
		    this);

gtk_window_set_title (GTK_WINDOW (last_check_window), "DCheck - Change Last Check");

gtk_window_set_default_size(GTK_WINDOW(last_check_window), 580, 350); // width, height

dtime_pixbuf	 = gdk_pixbuf_new_from_inline(-1,
                                            dtime_icon_pixbuf_src,
					                                  FALSE, NULL);
if (dtime_pixbuf) gtk_window_set_icon( GTK_WINDOW(last_check_window), dtime_pixbuf);

gtk_container_set_border_width(GTK_CONTAINER(last_check_window), 2);

vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_container_add (GTK_CONTAINER (last_check_window), vbox);

scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	 
//cout << "Last_check::change_last_check_gui - DONE gtk_scrolled_window_new()" << endl;
	



    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolled_window),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

	 
//cout << "Last_check::change_last_check_gui - DONE gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, ... " << endl;
	
// Add vertical box to scrolled window
box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
gtk_box_set_homogeneous(GTK_BOX(box), FALSE);
	
gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),	box);




// Add horizontal box to vertical box	
hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);

// Add 8 vertical boxes to horizontal box	
GtkWidget *column[9];
for (int n=0; n<9; ++n)
	{
	column[n] = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
	gtk_box_set_homogeneous(GTK_BOX(column[n]), TRUE); //(n<7) ? TRUE: FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), column[n], TRUE, TRUE, 0); // expand, fill, padding
	}

gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0); // expand, fill, padding


	
//----------
// HEADINGS
//----------

wid = gtk_label_new("Filesystem");
gtk_misc_set_alignment(GTK_MISC(wid), 0, 0.5);
gtk_box_pack_start(GTK_BOX(column[0]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Current Last Check");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[1]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Day");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[2]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Month");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[3]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Year");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[4]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("H");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[5]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("M");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[6]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("S");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[7]), wid, FALSE, FALSE, 0); // expand, fill, padding

wid = gtk_label_new("Now");
gtk_misc_set_alignment(GTK_MISC(wid), 0.5, 0.5);
gtk_box_pack_start(GTK_BOX(column[8]), wid, FALSE, FALSE, 0); // expand, fill, padding



	
for (i=Lists::FS_string_list.begin(); i!=Lists::FS_string_list.end(); ++i)

    {
		col = 0;
		fs_str = (FS_strings_info *)*i;
		FS_last_check *fs_lc = new FS_last_check;

	 	if (!fs_str) continue;
//cout << "Last_check::change_last_check_gui - Started iteration" << endl;
	

       
//------------
// FILESYSTEM
//------------
    wid = gtk_label_new( (fs_lc->fs_name=fs_str->name).c_str() );
    gtk_misc_set_alignment(GTK_MISC(wid), 0, 0.5);
		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(wid), FALSE, FALSE, 0); // expand, fill, padding


	 
//cout << "Last_check::change_last_check_gui - DONE s_lc->fs_name gtk_grid_attach" << endl;
	
                                                        

//-------------------
// CURRENT LAST CHECK
//-------------------

    fs_lc->last_check_label = gtk_label_new((fs_str->last_check).c_str());
    gtk_misc_set_alignment(GTK_MISC(fs_lc->last_check_label), 0.5, 0.5);
  	decorate_button(fs_lc->last_check_label, &brown, NULL, NULL, NULL);


// Convert last_check_txt to YYYYMMDD[HH[MM[SS]]] format
// Use strptime() to put the broken down time into tm
    memset(&tm, 0, sizeof(struct tm));
      
    strptime( (fs_str->last_check).c_str(), "%A %B %d %T %Y", &tm);
             // weekdayname monthname, decimaldayinmonth(OK), time(OK) decimalyear

		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(fs_lc->last_check_label), FALSE, FALSE, 0); // expand, fill, padding


//================
// NEW LAST CHECK
//================

//---------------
// DAY
//---------------
    fs_lc->day_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(fs_lc->day_entry), 2);
    gtk_entry_set_width_chars(GTK_ENTRY(fs_lc->day_entry), 2);
			
    gtk_entry_set_text(GTK_ENTRY(fs_lc->day_entry), convert_i2str(tm.tm_mday, 2).c_str());

    g_signal_connect(G_OBJECT(fs_lc->day_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)"D");
    gtk_widget_set_tooltip_text(fs_lc->day_entry,
                                    "Day\n"
                                    "Use format dd");
		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(fs_lc->day_entry), FALSE, FALSE, 0); // expand, fill, padding


//---------------
// MONTH
//---------------
    fs_lc->month_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(fs_lc->month_entry), 2);
    gtk_entry_set_width_chars(GTK_ENTRY(fs_lc->month_entry), 2);
			
    gtk_entry_set_text(GTK_ENTRY(fs_lc->month_entry), convert_i2str(tm.tm_mon+1, 2).c_str());

    g_signal_connect(G_OBJECT(fs_lc->month_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)"M");
			
    gtk_widget_set_tooltip_text(fs_lc->month_entry,
                                    "Month\n"
                                    "Use format mm");
 
		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(fs_lc->month_entry), FALSE, FALSE, 0); // expand, fill, padding


//---------------
// YEAR
//---------------     
    fs_lc->year_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(fs_lc->year_entry), 4);
    gtk_entry_set_width_chars(GTK_ENTRY(fs_lc->year_entry), 4);
    gtk_entry_set_text(GTK_ENTRY(fs_lc->year_entry), convert_i2str(tm.tm_year+1900, 4).c_str());

    g_signal_connect(G_OBJECT(fs_lc->year_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)"Y");
			
    gtk_widget_set_tooltip_text(fs_lc->year_entry,
                                    "Year\n"
                                    "Use format yyyy");

		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(fs_lc->year_entry), FALSE, FALSE, 0); // expand, fill, padding


//---------------
// HOUR
//---------------     
    fs_lc->hour_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(fs_lc->hour_entry), 2);
    gtk_entry_set_width_chars(GTK_ENTRY(fs_lc->hour_entry), 2);
    gtk_entry_set_text(GTK_ENTRY(fs_lc->hour_entry), convert_i2str(tm.tm_hour, 2).c_str());

    g_signal_connect(G_OBJECT(fs_lc->hour_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)"h");
    gtk_widget_set_tooltip_text(fs_lc->hour_entry,
                                    "Hour\n"
                                    "Use format hh");
		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(fs_lc->hour_entry), FALSE, FALSE, 0); // expand, fill, padding

//---------------
// MINUTE
//---------------     
    fs_lc->minute_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(fs_lc->minute_entry), 2);
    gtk_entry_set_width_chars(GTK_ENTRY(fs_lc->minute_entry), 2);

    gtk_entry_set_text(GTK_ENTRY(fs_lc->minute_entry), convert_i2str(tm.tm_min, 2).c_str());

    g_signal_connect(G_OBJECT(fs_lc->minute_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)"m");
    gtk_widget_set_tooltip_text(fs_lc->minute_entry,
                                    "Minute\n"
                                    "Use format mm");

		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(fs_lc->minute_entry), FALSE, FALSE, 0); // expand, fill, padding

//---------------
// SECOND
//---------------     
    fs_lc->second_entry = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(fs_lc->second_entry), 2);
    gtk_entry_set_width_chars(GTK_ENTRY(fs_lc->second_entry), 2);

    gtk_entry_set_text(GTK_ENTRY(fs_lc->second_entry), convert_i2str(tm.tm_sec, 2).c_str());

    g_signal_connect(G_OBJECT(fs_lc->second_entry), "insert_text",
		      G_CALLBACK(callback_insert_text_handler),
		      (gpointer)"s");

    gtk_widget_set_tooltip_text(fs_lc->second_entry,
                                    "Second\n"
                                    "Use format ss");

		gtk_box_pack_start(GTK_BOX(column[col++]),
		                   GTK_WIDGET(fs_lc->second_entry), FALSE, FALSE, 0); // expand, fill, padding

      
//---------------
// NOW
//---------------
		box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
		gtk_box_set_homogeneous(GTK_BOX(box), TRUE);

    fs_lc->now_check_button = gtk_check_button_new();
		gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(fs_lc->now_check_button), FALSE, FALSE, 0); // expand, fill, padding
		gtk_box_pack_start(GTK_BOX(column[col++]), GTK_WIDGET(box), FALSE, FALSE, 0); // expand, fill, padding
     
    g_signal_connect (G_OBJECT (fs_lc->now_check_button), "toggled",
		      G_CALLBACK (callback_now_check_button_toggled),
		      this);

		FS_last_check_list.push_back(fs_lc);

		} // End of    for (i=Lists::FS_string_list.begin(); .....




	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
  gtk_box_pack_end (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


  wid = gtk_button_new_with_mnemonic (" _Apply ");
  gtk_button_set_image(GTK_BUTTON(wid), gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON));

	decorate_button(GTK_WIDGET(wid), &darkgreen,  &lightyellow, NULL, NULL);        

	// Make sure the image is displayed!
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(wid)))); 

  g_signal_connect (G_OBJECT (wid), "clicked",
		    G_CALLBACK (callback_last_check_ok),  this);
   gtk_box_pack_start(GTK_BOX(hbox), 
		      wid, FALSE, TRUE, 0);
  

  // --- Create the "Cancel" button
  wid = gtk_button_new_with_mnemonic (" _Cancel ");
  gtk_button_set_image(GTK_BUTTON(wid), gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));  
  decorate_button(GTK_WIDGET(wid), &darkred,  &lightyellow, NULL, NULL);
// Make sure the image is displayed!
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(wid)))); 

  // --- Need to close the window if they press "Close"
  g_signal_connect (G_OBJECT (wid), "clicked",
		    G_CALLBACK (callback_last_check_cancel),  this);

  // --- Allow it to be the default button
//  GTK_WIDGET_SET_FLAGS (wid, GTK_CAN_DEFAULT);
	gtk_widget_set_can_default(wid, TRUE);

  // --- Add the button to the dialog
  gtk_box_pack_end(GTK_BOX(hbox), 
		      wid, FALSE, TRUE, 0);

  // --- Make the button the default button
  gtk_widget_grab_default (wid);

  // Show everything
  gtk_widget_show_all(last_check_window);

	gtk_main();


}
