/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       lists.cpp                                                 *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       01-Mar-2013 15.12.00 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Various lists                                                             *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "lists.h"

// Make this a list instead of a GList
//GList *Lists::FS_widget_list                   = NULL;

list<FS_widgets_info *> Lists::FS_widget_list;
list<FS_strings_info *> Lists::FS_string_list;
list<FS_control *>      Lists::FS_control_list;

Lists::Lists()
{
//cout << "Lists::lists" << endl;
}

Lists::~Lists()
{
//cout << "Lists::~lists" << endl;
}

/*******************************
 * Return TRUE if widget is    *
 * fs->interval_type_button in  *
 * list<FS_widgets_info        *
 *******************************/
bool Lists::is_fs_w_type(GtkWidget *widget)
{
list<FS_widgets_info *>::iterator i;

for (i=FS_widget_list.begin(); i!=FS_widget_list.end(); ++i)
	{
	if (	(widget == (((FS_widgets_info *)*i)->interval_type_button)) || 
				(widget == (((FS_widgets_info *)*i)->select_button))	)
		{
		return TRUE;
		}
	}
return FALSE;
}

/*************************************
 * Return the "FS_widgets_info *" of *
 * which this widget is a member     *
 *************************************/
FS_widgets_info *Lists::get_fs_w_of_widget(GtkWidget *widget)
{
	
list<FS_widgets_info *>::iterator i;


for (i=FS_widget_list.begin(); i!=FS_widget_list.end(); ++i)
	{			
	FS_widgets_info *fs = (FS_widgets_info *)*i;

  if (  (widget == fs->name_label)               ||
        (widget == fs->size_label)               ||
        (widget == fs->managed_check_button)     ||
        (widget == fs->dcheck_button)            ||
        (widget == fs->num_mounts_entry)         ||
        (widget == fs->max_mounts_entry)         ||
        (widget == fs->interval_entry)           ||
        (widget == fs->interval_type_button)     ||
        (widget == fs->select_button)      )
    {
    return fs;
    }
  }
return NULL;
}



/**************************************************
 * Get FS_widgets_info* from the filesystem name *
 **************************************************/
FS_widgets_info *Lists::get_fs_w(string name)
{
list<FS_widgets_info *>::iterator i;

for (i=FS_widget_list.begin(); i!=FS_widget_list.end(); ++i)
	{
			
	FS_widgets_info *fs = (FS_widgets_info *)*i;
	if (name == gtk_label_get_text(GTK_LABEL(fs->name_label)))
		{
    return fs;
    } 
  }
return NULL;
}

/*************************************
 * Return the "FS_widgets_info *" to *
 * which this  FS_strings_info refers*
 *************************************/
FS_widgets_info *Lists::get_fs_w(FS_strings_info *fs_str)
{
return get_fs_w(fs_str->name);
}

/*************************************
 * Return the "FS_strings_info *" of *
 * which  this name is a member      *
 *************************************/
FS_strings_info *Lists::get_fs_str(string name)
{
FS_strings_info *fs;
list<FS_strings_info *>::iterator i;

	
for (i=FS_string_list.begin(); i!=FS_string_list.end(); ++i)
	{
	fs = (FS_strings_info *)*i;
	if (fs->name == name) return fs;
	}
return NULL;
}

/*******************************************
 * Return the "FS_strings_info *" to which *
 * this  FS_widgets_info refers            *
 *******************************************/
FS_strings_info *Lists::get_fs_str(FS_widgets_info *fs_w)
{
return get_fs_str(gtk_label_get_text(GTK_LABEL(fs_w->name_label)));
}

void Lists::add_to_control_list(string name, string control)
{
string n = name;
string c = control;	
FS_control *fsc = new(FS_control);				
fsc->name = n;
fsc->control = c;
	
FS_control_list.push_back(fsc);

}


string Lists::get_control(string name)
{
FS_control *fsc;
list<FS_control *>::iterator i;
			
for (i=FS_control_list.begin(); i!=FS_control_list.end(); ++i)
  {
	fsc = (FS_control *)*i;
	if (fsc->name == name)
		{
		return fsc->control;
		}
	}
return (string)"";
}


/*
string name;                      // name e.g. /dev/sda2
string size;                      // size of this fs, B KiB MiB GiB Tib
string state;                     // state e.g. clean
string control;                   // dcheck if managed, otherwise tune2fs
string mount_point;               // last mount point
string last_check;                // when last checked
string num_mounts;                // current number of mounts
string max_mounts;                // maximum number of mounts before check
string interval;                  // maximum interval between checks
string interval_type;             // days, weeks or months or <none> if no checks
*/

//  For debug...  *************************************************************
void Lists::display_FS_string_list(void)
{
FS_strings_info *fs;
list<FS_strings_info *>::iterator i;

//cout << "\nmisc::display_FS_string_list:-" << endl;

if (FS_string_list.empty()) cout << "Dcheck::display_FS_string_list - list empty!!!" << endl;
	
for (i=FS_string_list.begin(); i!=FS_string_list.end(); ++i)
	{
	fs = (FS_strings_info *)*i;
		
	if (!fs)
		{
		//cout << "misc::display_FS_string_list - fs empty" << endl; 
		continue;
		}

	cout << fs->name << " : " << fs->size << " : " << fs->state << " : "; 
	cout << fs->control << " : " << fs->mount_point << " : " << fs->last_check << " : ";
	cout << fs->num_mounts << " : " << fs->max_mounts << " : " << fs->interval << " : "<< fs->interval_type << endl;
  }

//cout << "misc::display_FS_string_list - LEAVING" << endl;
}



//
void Lists::display_FS_control_list(void)
{
list<FS_control *>::iterator i;
FS_control *con;

cout << "Lists::display_FS_control_list - list:-"<< endl;
for (i=FS_control_list.begin(); i!=FS_control_list.end(); ++i)
	{
//	cout << "\nmisc::display_FS_control_list - GOT i" << endl;

	con = (FS_control *)*i;
	if (con->name.length())
		{
		cout << "misc::display_FS_control_list - name= " << con->name << " : control= " << con->control << endl;
		}
	}
cout << "Lists::display_FS_control_list - DONE"<< endl;
}

/*******************************************************************************/
