/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       lists.h                                                   *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       01-Mar-2013 14.54.53 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Header file for lists.cpp                                                 *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __LISTS_H__
#define __LISTS_H__

#include <list>
#include <iostream>

#include <gtk/gtk.h>


#include "misc.h"


using namespace std;

class  FS_widgets_info
{
public:
GtkWidget *name_label;           // name e.g. /dev/sda2
GtkWidget *size_label;           // size of this fs, B KiB MiB GiB Tib
GtkWidget *state_label;          // state e.g. clean
GtkWidget *managed_check_button; // dcheck if managed, otherwise tune2fs
GtkWidget *dcheck_button;        // tick it for check next boot
GtkWidget *num_mounts_entry;     // current number of mounts
GtkWidget *max_mounts_entry;     // ghosted if dcheck otherwise select maximum mounts before a check
string old_managed_num_mounts;   // previous value if managed is selected
string old_managed_max_mounts;   // previous value if managed is selected	
string old_dcheck_num_mounts;    // previous value if check is selected
string old_dcheck_max_mounts;    // previous value if check is selected	
GtkWidget *interval_entry;       // maximum interval between checks
GtkWidget *interval_type_button; // days, weeks or months or <none> if no checks
GtkWidget *select_button;	       // Select weeks or months or <none>
};


class FS_strings_info
{
public:
string name;                      // name e.g. /dev/sda2
string size;                      // size of this fs, B KiB MiB GiB Tib
string state;                     // state e.g. clean
string control;                   // dcheck if managed, otherwise tune2fs
bool check_wanted;                // TRUE if check wanted at next mount
string mount_point;               // last mount point
string last_check;                // when last checked
string num_mounts;                // current number of mounts
string max_mounts;                // maximum number of mounts before check
string interval;                  // maximum interval between checks
string interval_type;             // days, weeks or months or <none> if no checks
};



class FS_control
{
public:
string name;                      // name e.g. /dev/sda2
string control;                   // dcheck if managed, otherwise tune2fs
};



class Lists
{
public:
	Lists(void);
	~Lists(void);




	FS_strings_info *get_fs_str(string name);
	FS_strings_info *get_fs_str(FS_widgets_info *fs_w);
	
	FS_widgets_info *get_fs_w(string name);
	FS_widgets_info *get_fs_w(FS_strings_info *fs_str);
	
	FS_widgets_info *get_fs_w_of_widget(GtkWidget *widget);
	bool is_fs_w_type(GtkWidget *widget);

	void display_FS_string_list(void); // For debug
	void display_FS_control_list(void);// For debug

	void add_to_control_list(string name, string control);
	string get_control(string name);

	static list<FS_widgets_info *> FS_widget_list;        // A list of FS_widgets_info

	static list<FS_strings_info *> FS_string_list;        // A list of FS_strings_info

	static list<FS_control *>      FS_control_list;       // A list of FS_control from load_conf()


	
private:

protected:
	



};


#endif // __LISTS_H__
