/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       misc.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       22-Feb-2013 16.01.58 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Various functions used in common                                          *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

 
#include "misc.h"


namespace misc
{

/********************************
 * Return length of file fname  *
 * in bytes.                    *
 * Return FALSE if it fails.    *
 ********************************/

gboolean get_fsize(char *fname, size_t *size)
{
    struct stat statbuf;

    if (stat(fname, &statbuf) == 0)
		{
		*size = statbuf.st_size;
		return(TRUE);
		}
return(FALSE);
}

/******************************************
 * Pipe output of a command into a string *
 ******************************************/

string get_output(string command)
{
FILE *fp;
string output;;

char pipebuffer[PIPE_BUF];

if ( (fp = popen(command.c_str(), "r")) )
	{
	while ( fgets(pipebuffer, PIPE_BUF, fp) )
		{
    output += pipebuffer;
		}
  if (ferror(fp))
    {
    // ERROR CONDITION, returns empty string
    output.clear();
		}
	}

else
	  {
	  // ERROR: can't open pipe, returns empty string
	  }

pclose(fp);
return(output); // Either error or no output if empty...
}






/*************************************
 * Create a spinner (integer values) *
 *************************************/
GtkWidget *create_spinner(int s_start, int s_min, int s_max)
{
GtkAdjustment *spinner_adj;
spinner_adj = (GtkAdjustment *) gtk_adjustment_new (s_start, s_min, s_max, 1.0, 0.0, 0.0);
// creates the spinner, with no decimal places
return( gtk_spin_button_new (spinner_adj, 1.0, 0) );
}

/*************************************
 * Convert a long double to string   *
 * The string will ne changed by     *
 * subsequent calls                  *
 *************************************/
// I want to output a decimal string with no more that 2 chars after the decimal point...
char *convert_f_to_string(double val)
{
static char result[1024]; // should be long enough!

snprintf(result, 1024, "%.02f", val); // the size includes the NULL byte
return result;
}

/***************************************
 * Colourize a button                  *
 * fg bg prelight stock_id             *
 * Do nothing for NULL colour pointers *
 ***************************************/
void decorate_button(GtkWidget *widget, GdkRGBA *fg_normal, GdkRGBA *bg_normal,  GdkRGBA *bg_prelight, const gchar *stock_id)
{
if (fg_normal) gtk_widget_override_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, fg_normal); 
if (bg_normal) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, bg_normal);  
if (bg_prelight) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_PRELIGHT, bg_prelight);

if ((stock_id) && *(stock_id))
  {
  gtk_button_set_image(GTK_BUTTON(widget), gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON));
  // Make sure the image is displayed!
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(widget)))); 
  }
}









/*****************************
 * Get mount point from data *
 *****************************/
string get_mount_point(string data)
{
char *p, *t;
char c;
string mount_point;

// mount_point - Last mounted on:          /boot
string last_mount_point_txt = "Last mounted on:";
if ( (p = strstr((char *)data.c_str(), (char *)last_mount_point_txt.c_str())) )
  {
  p += 16;
  while ((*p==' ')||(*p=='\t')) p++;
  t = p;
  while ((*t)&&((*t)!='\n')) t++;
  c = *t;
  *t = 0;
  if ((*p) == 0)
		{
		mount_point = last_mount_point_txt + "\tNot mounted";
		}
  else mount_point = last_mount_point_txt + "\t" + p;
  *t = c;
  }
else
  {
  // ERROR
  mount_point = last_mount_point_txt + "\tUNKNOWN";
  }
return mount_point;
}


/*
 * Get last check from data
 */
string get_last_check(string data)
{
char *p, *t;
char c;

// last_check - Last checked:             Tue May 31 16:35:03 2011
string last_check_txt = "Last checked:";
string last_check;
  
if ( (p = strstr((char *)data.c_str(), (char *)last_check_txt.c_str())) )
  {
  p += 13;
  while ((*p==' ')||(*p=='\t')) p++;
  t = p;
  while ((*t)&&((*t)!='\n')) t++;
  c = *t;
  *t = 0;
  if ((*p) == 0)
		{
		last_check = last_check_txt + "\t\tNot checked yet" ;
		}
  else last_check = p;
  *t = c;
  }
else
  {
  // ERROR
  last_check = last_check_txt + "\t\tUNKNOWN";
  }

return last_check;
}


// Uses output of tune2fs -l XXXX
string get_os_type(string  data)
{
char *p, *t;
char c;
char *os_text = (char *)"Filesystem OS type:";
string os_type;

p = strstr( (char *)data.c_str(), os_text );
if (p)
  {
  p += 19;

  while ((*p==' ')||(*p=='\t')) p++;
  t = p;
  while ((*t)&&((*t)!='\n')) t++;
  c = *t;
  *t = 0;
  if ((*p) == 0) os_type = "UNKNOWN";
  *t = c;
  }
else
  {
  os_type = "UNKNOWN";
  }
return os_type;
}


string convert_i2str(int i, int pad)
{
ostringstream ost;
int p = (pad <=0 ) ? 1 : pad;
ost <<  setw(p) << setfill('0') << dec << i;
return ost.str();
}


} // End of namespace misc
