/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    dcheck                                                    *
*Project Version: 1.1.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Wednesday 6th March 2013                                  *
*                                                                           *
*Start DateStamp: 04-Nov-2011 17.42.21 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs/index.html                              *
*                                                                           *
*Project Summary:                                                           *
* Ask for a disk check of the selected filesystems at the next boot         *
* On restart, set them to not be checked until dcheck requests it again     *
* Also adjust  tune2fs for for other filesystems                            *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       status.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       20-Feb-2013 20.54.04 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Brings up a window to display output of commands                          *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "status.h"

GtkWidget *Status::status_window = NULL;
GdkPixbuf *Status::st_pixbuf = NULL;

GtkWidget *Status::status_box;

bool Status::status_wanted = TRUE;

Status::Status(void)
{
//cout << "Status::Status" << endl;
}
Status::~Status(void)
{
//cout << "Status::Status" << endl;
}

void Status::callback_status_destroy(GtkWidget *widget, Status *status)
{
if (st_pixbuf)
	{
	g_object_unref(G_OBJECT(st_pixbuf));
	st_pixbuf = NULL;
	}

status_window = NULL;
status_box    = NULL;
}

bool Status::status_running(void)
{
return( (status_window) );
}

void Status::callback_status_close(GtkWidget *widget, Status *status)
{
gtk_widget_destroy(GTK_WIDGET(status_window));
}

void Status::add_status_underline(void)
{	
add_status((string)"==========================================\n", FALSE);
}

void Status::add_status(string data, bool left_flag)
{
if (!status_wanted) return;
GtkWidget *label = gtk_label_new(data.c_str());
if (left_flag)
  {
  gtk_misc_set_alignment(GTK_MISC(label), 0, .5);
  }
gtk_box_pack_start(GTK_BOX(status_box), label, TRUE, TRUE, 0);
gtk_widget_show (label);
}

void Status::fix_status(string data, bool left_flag)
{
if (!status_wanted) return;
if (!status_window)
  {
  GtkWidget *scrolled_window;
  GtkWidget *vbox, *hbox;
  GtkWidget *button;


  status_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_show (status_window);
  g_signal_connect (G_OBJECT (status_window), "destroy",
		    G_CALLBACK (callback_status_destroy),
		    NULL);

  gtk_window_set_title (GTK_WINDOW (status_window), "Dcheck - Status");

	// This opens with a very small size... resize is necessary
  gtk_window_set_default_size(GTK_WINDOW(status_window), 500, 350); // width, height
	gtk_window_resize(GTK_WINDOW(status_window), 500, 450); // width, height

  gtk_container_set_border_width(GTK_CONTAINER(status_window), 2);

  if (!st_pixbuf)
    {
    st_pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(status_window),
				  GTK_STOCK_ABOUT,
				  GTK_ICON_SIZE_DIALOG);
    }
  if (st_pixbuf) gtk_window_set_icon( GTK_WINDOW(status_window), st_pixbuf);

  gtk_window_set_keep_above(GTK_WINDOW(status_window), TRUE);

	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);

  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (status_window), vbox);


  scrolled_window = gtk_scrolled_window_new(NULL, NULL);

    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
  gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolled_window),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

  gtk_widget_show (scrolled_window);
  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

  status_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);

//	gtk_button_box_set_layout(GTK_BUTTON_BOX(status_box), GTK_BUTTONBOX_EDGE);   //GTK_BUTTONBOX_SPREAD), 
  gtk_widget_show (status_box);

  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
					status_box);

  add_status(RH_VERSION_STRING, FALSE);
	add_status_underline();
  add_status(data, TRUE);  // command
		
	hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_EDGE);// GTK_BUTTONBOX_SPREAD);
	gtk_box_set_spacing(GTK_BOX(hbox), 0);

  gtk_box_pack_end (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  // --- Create the "Close" button
  button = gtk_button_new_with_mnemonic (" _Close "); // Use Alt+c
  gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));  
  misc::decorate_button(GTK_WIDGET(button), &misc::darkred,  &misc::lightyellow, NULL, NULL);
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(button)))); 

  // --- Need to close the window if they press "Close"
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_status_close),  NULL);

  // --- Allow it to be the default button
  // GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_set_can_default(button, TRUE);

  // --- Add the button to the dialog
  gtk_box_pack_end(GTK_BOX(hbox), 
		      button, FALSE, TRUE, 0);

  // --- Make the button the default button
  gtk_widget_grab_default (button);

  // --- Make the button visible
  gtk_widget_show_all (status_window);
  }
else add_status(data, left_flag);
}
